/* definitions for standard programs */



#include <stdlib.h>

#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <string.h>

#include <ctype.h>

#include <nwtypes.h>

#include <process.h>

#include <nwenvrn.h>



#define PC31_DATA 0x300        /* data port */

#define PC31_STAT PC31_DATA+1  /* status port */

#define GETDATA 0              /* read command for clock data */

#define TIMEOUT 100            /* 100 ms timeout for status to stay busy */

#define QUARTZ 1

#define QUARTER_SEC 250        /* 1/4 sec */



struct {

       BYTE Sec100;

       BYTE Second;

       BYTE Minute;

       BYTE Hour;

       BYTE DayOfMonth;

       BYTE DayOfWeek;

       BYTE Month;

       BYTE Year;

       BYTE Status;

       } clockdata;



/* Read data packet from clock. First the command byte is written to the status

   port and status is read until busy flag is clear or timeout reached. If busy

   is clear the structure clockdata is filled with data bytes read from the data

   port. This function returns 0 on success or -1 if timeout occured */



int readPC31(void)

{

int i;

BYTE *p;



outp(PC31_STAT,GETDATA);

for (i=0;i<TIMEOUT;i++)

  if (inp(PC31_STAT) & 1) /* busy */

    delay(1);

  else

    break;

if (i==TIMEOUT)

  return(-1);



for (i=0,p=(BYTE *)&clockdata;i<sizeof(clockdata);i++,p++)

  *p=inp(PC31_DATA);

if (clockdata.Status & QUARTZ)

  clockdata.Status='Q';

else

  clockdata.Status='D';

return(0);

}



/* Read clock every 1/4 sec and set server time every minute. Also the current

   time from the DCF77 clock is displayed as <status>: <hour>:<min>.sec where

   <status> is 'D' if clock is synchronized with DCF77 or 'Q' if clock runs

   free on its own quartz oscillator */



void main(int argc, char *argv[])

{

int clockUpdated=0,run=1;



if (readPC31())

  {

  printf("Error reading clockdata\n");

  return;

  }

else

  {

  SetAutoScreenDestructionMode(TRUE);

  printf("DCF77 clock support Version 1.0 (c) 1992 by Peter Heitzer\n\n");

  printf("Press <ESC> key to stop program\n");

  do {

     if (!readPC31())

       {

       printf("\r%c: %02d:%02d.%02d",clockdata.Status,clockdata.Hour,clockdata.Minute,clockdata.Second);

       if (!clockdata.Second) /* full minute */

         {

         if (!clockUpdated)

             {

             SetFileServerDateAndTime(

             clockdata.Year,

             clockdata.Month,

             clockdata.DayOfMonth,

             clockdata.Hour,

             clockdata.Minute,

             clockdata.Second );

             clockUpdated=1;

             }

           }

         else

           clockUpdated=0;

         }

     delay(QUARTER_SEC);

     if (kbhit())

       run=getch()!=27;

     } while (run);

  }

}

