#include <stdio.h>

#include <stdlib.h>

#include <string.h>

#include <conio.h>

#include <errno.h>

#include <ctype.h>

#include <nwqueue.h>

#include <nwbindry.h>

#include <niterror.h>

#include <nwdir.h>

#include <process.h>

#include <time.h>

#include <nwmisc.h>

#include <nwenvrn.h>

#include <nwconn.h>

#include <io.h>

#include <nwaccntg.h>



#define MAXQ 8

#define VERSION "Filtering Printserver 1.00"

#define CONFIG "SYS:SYSTEM/FILTERPS.CFG"   /* parameter file name */

#define MAXROW 24

#define ONESEC 1000

#define REFRESH 5000

#define BEEP putchar(7)

#define CLREOL printf("%-78s\r"," ")



int myScreen,  /* screen handle */

    run=FALSE;



typedef enum { FLT_NONE, FLT_PS_ONLY } FILTER;



typedef struct {

               char sourceQueueName[48];

               long sourceQueue;

               long processed;

               long rejected;

               long charge;

               FILTER filter;

               char targetQueueName[48];

               long targetQueue;

               } QINFO;



/* Functions for loading parameters from file

   Format of parameter file:

   #QUEUE=<sourcequeue>,<targetqueue>,<filtertype>,<charge>

   any line which starts not with #QUEUE or #REFRESH is ignored */



#define KEY_QUEUE "QUEUE"

#define DELIM "\n\r\t, "

#define F_NONE "NONE"

#define F_PS_ONLY "PSONLY"



/* load parameter from file */



int loadParameter(char *filename,QINFO *q,int *qNum)

{

char tmp[128],*qs,*qt,*ft,*ch,*p;

long sID,tID;

FILE *param;

if ((param=fopen(filename,"r"))==NULL)

  return(-1);

while (!feof(param)) {

  if (fgets(tmp,128,param)==NULL)

    break;

  if (*qNum==MAXQ)

    break;

  if (tmp[0]=='#')  /* parameter ? */

    if (p=strtok(tmp+1,"="))

      {

      ft=ch=NULL;

      if (!stricmp(p,KEY_QUEUE))

        {

        if (qs=strtok(NULL,DELIM))

          if (qt=strtok(NULL,DELIM))

            {

            if (ft=strtok(NULL,DELIM))

              ch=strtok(NULL,DELIM);

            if (!GetBinderyObjectID(qs,OT_PRINT_QUEUE,&sID))

              if (!GetBinderyObjectID(qt,OT_PRINT_QUEUE,&tID))

                {

                ++*qNum;

                q->sourceQueue=sID;

                q->targetQueue=tID;

                q->processed=q->rejected=0;

                strcpy(q->sourceQueueName,qs);

                strcpy(q->targetQueueName,qt);

                q->charge= ch ? atoi(ch) : 0;

                q->filter=FLT_NONE;

                if (ft)

                  {

                  if (!stricmp(ft,F_NONE))

                    q->filter=FLT_NONE;

                  else

                    if (!stricmp(ft,F_PS_ONLY))

                      q->filter=FLT_PS_ONLY;

                  }

                q++;

                }

            }

        }

      }

    }

fclose(param);

return(0);

}



void qstatus(QINFO *q)

{

printf("%-16s ->: %-16s, (%-16s charge: %ld)\n",q->sourceQueueName,q->targetQueueName,

        (q->filter==FLT_NONE) ? "No filter" : "PS only",q->charge);

printf("  Processed: %08ld,  rejected: %08ld",q->processed,q->rejected);

}



/* read password from console */



char *readpass(void)

{

static char pass[48]="\0";

int i=0;



while (i<48)

  {

  switch (pass[i]=getch()) {

    case 3: return(NULL);

    case 8: if (!i)

              putchar(7);

            else

              {

              printf("\b \b");

              i--;

              }

            break;

    case 13 : pass[i]=0; i=48; break;

    default : if (isgraph(pass[i]))

                {

                i++;

                putchar('_');

                }

              break;

    }

  }

printf("\n");

return(strupr(pass));

}



void filterThread(QINFO *q)

{

int inHandle,outHandle,inNum,outNum;

NWQJobRec_t inJob,outJob;

BYTE buf[1024];

long n,packets;

char *p;

WORD oT;



do {

   if (!NWQBeginJobService(q->sourceQueue,0xffff,&inJob,&inHandle))

     {

     q->processed++;

     packets=0;

//     memcpy(&outJob,&inJob,sizeof(NWQJob));

     outJob=inJob;

     outJob.job.controlFlags=0;

     inNum=inJob.job.num;

     NWQChangeToClientRights(q->sourceQueue,inNum);



     if (!NWQCreateJob(q->targetQueue,&outJob,&outHandle))

       {

       outNum=outJob.job.num;

       do {

          if ((n=read(inHandle,buf,1024))==-1l)

            break;

          if (!packets)

            if (q->filter==FLT_PS_ONLY)

              {

              p=strtok((char*)buf,"\004\n\r");

              if (p==NULL)

                break;

              if (strncmp(p,"%!",2))

                break;

              }

          if (n==0)

            break;

          write(outHandle,buf,n);

          packets++;

          } while (1);

       if (!packets)

         {

         ConsolePrintf("Rejecting job %d (%s)\n",IntSwap((WORD)inNum),inJob.textJobDescription);

         NWQAbortJob(q->targetQueue,outJob.job.num,outHandle);

         q->rejected++;

         }

       else

         NWQMarkJobForService(q->targetQueue,outNum,outHandle);

       }

     else

       q->rejected++;

     NWQRestoreServerRights();

//     ConsolePrintf("Charge for job %d: %d\n",IntSwap((WORD)inNum),packets ? q->charge : NULL);

     if (packets && q->charge)

       {

       GetBinderyObjectName(inJob.client.id,(char *)buf,&oT);

       SubmitAccountCharge(oT,(char *)buf,OT_PRINT_SERVER,q->charge,NULL,

                           0x8000,inJob.textJobDescription);

       }

     NWQEndJobService(q->sourceQueue,inNum,packets ? q->charge : NULL,inHandle);

     }

   delay(1000);

   } while (run);

}



void main(int argc,char *argv[])

{

int     queueCnt=0,i,cC,errors=0;

char    screenName[80],*config=CONFIG,*ps,*pass;

QINFO   *q,qinfo[MAXQ];

long    psID;



sprintf(screenName,"%s Screen",VERSION);

if ((myScreen=CreateScreen(screenName,AUTO_DESTROY_SCREEN))==EFAILURE)

  return;

DisplayScreen(myScreen);



if (argc==1)

  {

  printf("Usage: LOAD %s PSNAME [configfile]\n",argv[0]);

  return;

  }



if (argc>2)

  config=strupr(argv[2]);

ps=strupr(argv[1]);



loadParameter(config,qinfo,&queueCnt);



if (!queueCnt)

  {

  printf("No queues found\n");

  PressAnyKeyToContinue();

  return;

  }

qsort(qinfo,queueCnt,sizeof(QINFO),stricmp);



if (GetBinderyObjectID(ps,OT_PRINT_SERVER,&psID))

  {

  printf("No print server named %s found\n",ps);

  return;

  }

printf("Enter password for print server %s: ",ps);

pass=readpass();



if (cC=LoginToFileServer(ps,OT_PRINT_SERVER,pass))

  {

  printf("LoginToFileServer returned error %04x\n",cC);

  PressAnyKeyToContinue();

  return;

  }

atexit(Logout);



for (i=0;i<queueCnt;i++)

  {

  if  (cC=NWQAttachServer(qinfo[i].sourceQueue))

    {

    printf("NWQAttachServer returned error %4x for queue %s\n",cC,

           qinfo[i].sourceQueueName);

    errors++;

    }

  }



for (i=0;i<queueCnt;i++)

  if (BeginThread(filterThread,NULL,8192,&qinfo[i])==EFAILURE)

    {

    printf("Error creating thread for queue %s\n",qinfo[i].sourceQueueName);

    PressAnyKeyToContinue();

    return;

    }



if (errors)

  {

  PressAnyKeyToContinue();

  return;

  }



run=TRUE;



do {

  CheckIfScreenDisplayed(myScreen,TRUE);  /* wait until screen is displayed */

  clrscr();

  printf("%s Main Screen, config: %s\n\n",VERSION,config);

  for (i=0,q=qinfo;i<queueCnt;i++,q++)

    {

    gotoxy(0,2+i*2);

    qstatus(q);

    }

  gotoxy(0,MAXROW);

  delay(REFRESH);

  } while (run);

}

