/* definitions for standard programs */



#include <stdlib.h>

#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <string.h>

#include <ctype.h>

#include <process.h>

#include <fshooks.h>

#include <fileengd.h>

#include <nwmisc.h>

#include <nwconn.h>



#define MAXNODE 1024



LONG hPreOpenFile=0,hPreGenOpenFile=0,hPostOpenFile=0;

typedef BYTE NODE[6];



NODE nodes[MAXNODE];

int nodeCnt=0;



int nodecmp(const void *node1, const void *node2)

{

return memcmp(node1,node2,sizeof(NODE));

}



int nodeOK(BYTE *n)

{

return bsearch(n,nodes,nodeCnt,sizeof(NODE),nodecmp)!=NULL;

}



char *nodestr(BYTE *n)

{

static char buf[16];

sprintf(buf,"%02X%02X%02X%02X%02X%02X",n[0],n[1],n[2],n[3],n[4],n[5]);

return buf;

}



/* read bootconf.sys and read all node addresses in it */



int readBootconf(void)

{

FILE *f;

char buf[256],tmp[3],*p;

int i;

if ((f=fopen("SYS:LOGIN\\BOOTCONF.SYS","r"))==NULL)

  return 1;

for (tmp[2]=0;nodeCnt<MAXNODE;) {

  if (fgets(buf,256,f)==NULL)

    break;

  strupr(buf);

  if (strncmp(buf,"0X",2))

    continue;

  if (p=strchr(buf+2,','))

    if (p=strtok(p+1,"="))

      if (strlen(p)==12)

        {

        for (i=0;i<6;i++,p+=2)

          {

          strncpy(tmp,p,2);

          nodes[nodeCnt][i]=(BYTE)strtoul(tmp,NULL,16);

          }

        nodeCnt++;

        }

  }

fclose(f);

if (nodeCnt==0)

  return 2;

qsort(nodes,nodeCnt,sizeof(NODE),nodecmp);

return 0;

}



void cleanup(void)

{

if (hPreOpenFile)

  NWRemoveFSMonitorHook(FSHOOK_PRE_OPENFILE,hPreOpenFile);

if (hPreGenOpenFile)

  NWRemoveFSMonitorHook(FSHOOK_PRE_GEN_OPEN_CREATE,hPreGenOpenFile);

if (hPostOpenFile)

  NWRemoveFSMonitorHook(FSHOOK_POST_OPENFILE,hPostOpenFile);

}



/* check if executable is opened, i.e. last part ends with .exe or .com */



int isEXE(char *fn,int compCount)

{

int i;



if (fn==NULL)

  return FALSE;

for (i=0;i<compCount-1;i++)

  {

  fn+=(*fn+1);

  }

if ((i=*fn)>4)

  {

  fn+=(i-3);

  if (!strnicmp(fn,".EXE",4) || !strnicmp(fn,".COM"))

    return TRUE;

  }

return FALSE;

}



void getFullPath(OpenFileCallBackStruct *ofstr,

                 GenericOpenCreateCBStruct *gofstr,

                 char *fn)



{

LONG   volume;

LONG   dirBase;

LONG   pathComponentCount,pathCount;

BYTE   *s,*d,*pathString,pathString0[256];

int    i,n;



if (ofstr)

  {

  volume=ofstr->volume;

  dirBase=ofstr->dirBase;

  pathComponentCount=ofstr->pathComponentCount;

  pathString=ofstr->pathString;

  }

else

  if (gofstr)

    {

    volume=gofstr->volume;

    dirBase=gofstr->dirBase;

    pathComponentCount=gofstr->pathComponentCount;

    pathString=gofstr->pathString;

    }

  else

    return;

d=fn;



if (fn==NULL)

  return;

if (!FEMapVolumeAndDirectoryToPath(volume,dirBase,pathString0,&pathCount))

  {

  if (pathString0)

    for (i=0,s=pathString0;i<pathCount;i++)

      {

      n=*s++;

      strncpy(d,s,n);

      d+=n;

      *d++='\\';

      *d=0;

      s+=n;

      }

  };

if (pathString)

  for (i=0,s=pathString;i<pathComponentCount;i++)

    {

    n=*s++;

    strncpy(d,s,n);

    d+=n;

    if (i<pathComponentCount-1)

      *d++='\\';

    *d=0;

    s+=n;

    }

}



int hookPreOpenFile(OpenFileCallBackStruct *ofstr)

{

static char fn[256],*s;

int l;

BYTE node[16],net[16];



*fn=0;

if (!isEXE(ofstr->pathString,ofstr->pathComponentCount))

  return 0;



getFullPath(ofstr,NULL,fn);



if (*fn)

    {

    GetInternetAddress(ofstr->connection,net,node);

    printf("\nConnection: %4ld %s '%s'",ofstr->connection,

    nodestr(node),fn);

    if (l=nodeOK(node))

      printf(" allow");

    else

      printf(" deny");

    return l ? 0 : 5;

    }

return 0;

}



int hookPreGenOpenFile(GenericOpenCreateCBStruct *ofstr)

{

static char fn[256],*s;

int l;

BYTE node[16],net[16];



*fn=0;

if (!isEXE(ofstr->pathString,ofstr->pathComponentCount))

  return 0;



getFullPath(NULL,ofstr,fn);



if (*fn)

    {

    GetInternetAddress(ofstr->connection,net,node);

    printf("\nConnection: %4ld %s '%s'",ofstr->connection,

    nodestr(node),fn);

    if (l=nodeOK(node))

      printf(" allow");

    else

      printf(" deny");

    return l ? 0 : 5;

    }

return 0;

}



int hookPostOpenFile(OpenFileCallBackStruct *ofstr,LONG *cC)

{

return 0;

}



void main(int argc, char *argv[])

{

LONG cC;

if (cC=readBootconf())

 {

 printf("Error %d reading bootconf.sys\n",cC);

 return;

 }

ConsolePrintf("\nFSHOOK: %ld nodes read.\n",nodeCnt);

atexit(cleanup);

AtUnload(cleanup);



if (cC=NWAddFSMonitorHook(FSHOOK_PRE_OPENFILE,hookPreOpenFile,&hPreOpenFile))

  {

  printf("NWAddFSMonitorHook error %08lx\n",cC);

  exit(0);

  }

if (cC=NWAddFSMonitorHook(FSHOOK_PRE_GEN_OPEN_CREATE,hookPreGenOpenFile,&hPreGenOpenFile))

  {

  printf("NWAddFSMonitorHook error %08lx\n",cC);

  exit(0);

  }

 if (cC=NWAddFSMonitorHook(FSHOOK_POST_OPENFILE,hookPostOpenFile,&hPostOpenFile))

  {

  printf("NWAddFSMonitorHook error %08lx\n",cC);

  return;

  }

ExitThread(TSR_THREAD,0);

}

