/*

     GRANTD.C - Dynamically create and delete scratch directories

     upon user log in and out.



     Copyright (c) 1993

     Ronald W. Butts

     The Wichita State University

     Wichita, KS 67260-0044



     butts@shocker.ee.twsu.edu



     NetWare 4.x support by Peter Heitzer

     peter.heitzer@rz.uni-regensburg.de



*/



#include <ctype.h>

#include <stdlib.h>

#include <stdio.h>

#include <stddef.h>

#include <fcntl.h>

#include <conio.h>

#include <direct.h>

#include <nwfile.h>

#include <nwdir.h>

#include <string.h>

#include <advanced.h>

#include <process.h>

#include <niterror.h>

#include <errno.h>

#include <nwconn.h>

#include <nwdir.h>

#include <io.h>

#include <nwenvrn.h>

#include <nwmisc.h>

#include <time.h>



#include <nwdsapi.h>

#include <nwcntask.h>



// Definitions of file and directory entry attribute bits



#define   ATTRIB_RO      0x1

#define   ATTRIB_H       0x2

#define   ATTRIB_SYS     0x4

#define   ATTRIB_XO      0x8

#define   ATTRIB_DIR     0x10

#define   ATTRIB_A       0x20

#define   ATTRIB_SH      0x80

#define   ATTRIB_T       0x1000

#define   ATTRIB_RA      0x4000

#define   ATTRIB_WA      0x8000

#define   ATTRIB_P       0x10000

#define   ATTRIB_RI      0x20000

#define   ATTRIB_DI      0x40000

#define   ATTRIB_CI      0x100000



#define MAX_EVT 50

#define MAX_SPECIAL 512

#define SPECIAL_FILE "SYS:ETC\\GRANTD.DAT"



typedef struct {

        char owner[16];

        long quota;

        long maxAge;

        } SPECIAL;



SPECIAL special[MAX_SPECIAL];

int specialCount=0;



unsigned trapInPtr,trapOutPtr,threadState,scratchVolumeIsMounted=0;

unsigned trapConnection[MAX_EVT],trapType[MAX_EVT];

char trapUser[MAX_EVT][128];



LONG loginEventHandle,logoutEventHandle,mountEventHandle,dismountEventHandle;

int myThreadID,scratchVolumeNumber,startNow=1,debugFlag=0;

VOLUME_STATS scratchVolumeStatistics;

char scratchDir[256];

long maxAge=24,lastDT=0;

NWDSContextHandle   dContext;



char *timeStr(void);



/* get local object ID, replaces GetBinderyObjectID */



LONG myGetLocalObjectID(char *oName)

{

WORD connID;

LONG oID=NULL;

int cC;



SetCurrentConnection (-1);

connID = GetCurrentConnection();

cC=NWDSMapNameToID(dContext,connID,oName,&oID);

ReturnConnection(connID);

return cC ? NULL : oID;

}



/* map local object ID to object name, replacement for GetBinderyObjectName */



char *myMapLocalObjectIDToName(LONG ID)

{

WORD connID;

static char name[256];

int cC;



SetCurrentConnection (-1);

connID = GetCurrentConnection();

cC=NWDSMapIDToName(dContext,connID,ID,name);

ReturnConnection(connID);

return cC ? "??" : name;

}



/* check modification date of special file and update data if it has changed */



int updateSpecial(void)

{

DIR *dp,*de;

int rc;

long dt;

FILE *sp;

char line[80],*p1,*p2,*p3,*p4;



if ((dp=opendir(SPECIAL_FILE))==NULL)

  {

  return 0;

  }

de=readdir(dp);

if (de==NULL)

  rc=0;

else

  {

  dt=((de->d_date)<<16)|de->d_time;

  if (dt!=lastDT)

    {

    rc=1;

    lastDT=dt;

    }

  else

    rc=0;

  }

closedir(dp);



if (sp=fopen(SPECIAL_FILE,"r"))

  {

  specialCount=0;

  while (fgets(line,80,sp)!=NULL) {

    strupr(line);

    if (p1=strtok(line," \n"))

      if (p2=strtok(NULL," \n"))

        if (p3=strtok(NULL," \n"))

          {

          p1[15]=0;

          strcpy(special[specialCount].owner,p1);

          special[specialCount].quota=strtoul(p2,&p4,10);

          if (*p4=='M')

            special[specialCount].quota<<=8;  // quota is MB

          else

            special[specialCount].quota=(special[specialCount].quota+3)>>2;  // quota is KB

          special[specialCount].maxAge=atoi(p3);

          specialCount++;

          if (specialCount==MAX_SPECIAL)

            break;

          }

    }

  fclose(sp);

  qsort(special,specialCount,sizeof(SPECIAL),stricmp);

  }

return rc;

}



unsigned WhatIsThisPath(char * path)

{

     /*   Given a Netware path, this function returns 0 if neither directory

          nor file; 1 if it is a file; 2 if it is a directory.             */



     DIR * dirPtr, *direntPtr;



     if(access(path,0))

          return 0;      // It does not exist

     if( (dirPtr=opendir(path)) ==NULL)

          return 0;

     direntPtr=readdir(dirPtr);

     if (direntPtr==NULL)

       {

       closedir(dirPtr);

       return 0;

       }

     if(!(direntPtr->d_attr & ATTRIB_DIR))

     {

          closedir(dirPtr);

          return 1;

     }

     else

     {

          closedir(dirPtr);

          return 2;

     }

}



int FlagEntryAsNPurge(char * path, unsigned purgeFlag)

{

     /*   This function takes a Netware directory and flags it as

          Normal + optionally P if purgeFlag is non-zero              */



     struct ModifyStructure modStruct;

     unsigned retCode;



              // Alter all defined bits except X and Subdir.

     modStruct.MFileAttributesMask=

          ATTRIB_RO | ATTRIB_H | ATTRIB_SYS | ATTRIB_A | ATTRIB_SH |

          ATTRIB_T | ATTRIB_RA | ATTRIB_WA | ATTRIB_P | ATTRIB_RI|

          ATTRIB_DI | ATTRIB_CI;



     if (purgeFlag)

          modStruct.MFileAttributes=ATTRIB_P;   // set P bit to 1

     else

          modStruct.MFileAttributes=0x00000000;   // set P bit to 0

     retCode= ChangeDirectoryEntry(path,&modStruct,MFileAttributesBit,0);

     return retCode;

}



/* return true if directory is empty */



unsigned emptyDir(char * path)

{

     char *tempPath;

     unsigned pathType;

     DIR * dirPtr;



     pathType=WhatIsThisPath(path);



     if(!pathType)

          return 0;           // Exit now if it does not exist.

     else if(pathType==1)

          return 0;           // Also exit now if not a directory.



     if((tempPath=malloc(256))==NULL)

          return 0;



     sprintf(tempPath,"%s\\*.*",path);



     // First open the directory for scanning; exit if it is empty



     if((dirPtr=opendir(tempPath))==NULL)

     {

          free(tempPath);

          return 1;

     }



     closedir(dirPtr);                         // done scanning directory

     free(tempPath);

     return 0;

}



unsigned KillSubTree(char * path, int leaveParent)

{

     /*   This function takes a Netware directory (do not end with \) and

          obliterates the files it contains as well all subdirectories and

          their files. The specified directory entry itself is not removed,

          it is simply left empty. All subdirectories are flagged as P before

          the files are removed, but the initial directory's P status is not

          altered.                                                    */



     char *tempPath;

     unsigned retCode,pathType;

     DIR * dirPtr, * direntPtr;



     pathType=WhatIsThisPath(path);



     if(!pathType)

          return 1;           // Exit now if it does not exist.

     else if(pathType==1)

          return 2;           // Also exit now if not a directory.



     if((tempPath=malloc(256))==NULL)

          return 3;



     sprintf(tempPath,"%s\\*.*",path);



     // First open the directory for scanning; exit if it is empty



     if((dirPtr=opendir(tempPath))==NULL)

     {

          free(tempPath);

          return 0;

     }



     // Next, kill all the files that I can get with simple kill--fast.



     remove(tempPath);



     // Now, ferret out the special files and directories that remain



     while( (direntPtr=readdir(dirPtr)) !=NULL)

     {

          sprintf(tempPath,"%s\\%s",path,direntPtr->d_name);



          FlagEntryAsNPurge(tempPath,1);       // Whatever it is, make it normal+P



          if(direntPtr->d_attr & ATTRIB_DIR)    // Is entry a directory?

          {

               ThreadSwitch();                 // Be a nice little NLM and

                                               // relinquish control every dir

                                               // level.

               KillSubTree(tempPath,0);

               rmdir(tempPath);

          }

          else

               retCode=remove(tempPath);       // nice knowin' ya, Mr. File.

     }

     closedir(dirPtr);                         // done scanning directory

     free(tempPath);

     if (!leaveParent)

       rmdir(path);  // 15.5.97 PH

     return 0;

}



void PrintHelp()

{

     puts("\nGRANTD usage: GRANTD -D base_dir [-A max_age ] {-P} {-C} {-Q quota} \n\t{-T rights_list} {-V}");

     puts("Where:");

     puts("  -D = Define base directory in which the work directories are to");

     puts("       be created. E.g., say -D SYS:SCRATCH");

     puts("  -P = Flag created directory with Purge attribute.");

     puts("  -C = Remove empty directory when user logs out\n");

     puts("  -A = Enable autodeletion of temp dirs and set maximum age to max_age hours.");

     puts("  -Q = Set directory space limit to quota when user logs in. Default");

     puts("       is unlimited directory space. E.g. -Q 2048 sets a 2 MB limit.");

     puts("       If quota ends with 'M' use MB else use KB.");

     puts("  -T = Define trustee rights for created directory. Valid letters are");

     puts("       RWCEMFAS. Default rights are all except AS. E.g. -T RWCEMFA");

     puts("  -V = Verbose mode");

     puts("\n");

}



int ZapDirectoryTrustees(char * path)

{

     /*   This function takes a Netware path and removes all trustees from

          the directory. Actually, it stops at 32, since the Novell API

          documents "forgot" to tell me how to get subsequent globs of 32

          trustees.                                                        */



     LONG trustees[32],numberTrustees;

     WORD masks[32], counter;

     int retCode;

     char CWDBuffer[128];



     getcwd(CWDBuffer,128);        // CWD is requested just for error text



     retCode=ScanTrustees(path,0,32,&trustees[0],&masks[0],&numberTrustees);



     if(retCode && retCode!=0x9c)  // Is it any error except "No more trustees?"

     {

          printf("** Error %X in ScanTrustees() for \"%s\" in dir %s\n",retCode,path,CWDBuffer);

          return retCode;

     }



     if(!retCode)   // if function above found no trustees, skip the loop.

          for(counter=0;counter< (WORD) numberTrustees;counter++)     // Remove them all

          {

               if((retCode=DeleteTrustee(path,trustees[counter])))

               {

                    printf("** Error %X in DeleteTrustee(%lX) for \"%s\" in dir %s\n",retCode,trustees[counter],path,CWDBuffer);

                    return retCode;

               }

          }



     return 0;

}



int MakeDirectory(char * path, LONG newOwner, unsigned rightsMask, int spaceLimit,int purgeFlag)

{

     /*   This function takes a Netware path, empties it of any previous

          trustees (or simply creates an empty directory if it

          does not already exist). Then it grants object newOwner the selected

          trustee rights to the directory. If purgeFlag is non-zero, the

          directory is flagged N+P, else flagged N. Lastly, the directory

          quota for the directory is set to the specified value.      */



     int retCode;

     char CWDBuffer[128];

     struct ModifyStructure mVector;



     getcwd(CWDBuffer,128);

     mkdir(path);                  // Create path (quietly fails if already exists)



          // Now flag it as Normal + maybe Purge in case it exists and is funky.



     if( (retCode=FlagEntryAsNPurge(path,purgeFlag)) )

          return retCode;



     if((retCode=ZapDirectoryTrustees(path)))

          {

          printf("** Error %X in ZapDirectoryTrustees() for \"%s\" in dir %s\n",retCode,path,CWDBuffer);

          return retCode;

          }

     if((retCode=AddTrustee(path,newOwner,rightsMask)))     // add single trustee

     {

          printf("** Error %X in AddTrustees(%lX) for \"%s\" in dir %s\n",retCode,newOwner,path,CWDBuffer);

          return retCode;

     }



     if (spaceLimit)

       if((retCode=AddSpaceRestrictionForDirectory(path,spaceLimit,0)))

       {

          printf("** Error %X in AddSpaceRestriction(%s,%d,0) for \"%s\" in dir %s\n",retCode,

          path,spaceLimit,path,CWDBuffer);

          return retCode;

       }

     mVector.MOwnerID=LongSwap(newOwner);

     if (retCode=ChangeDirectoryEntry (path,&mVector,MOwnerIDBit,0))

       {

       printf("** Error %X setting owner of %s\n",path);

       return retCode;

       }

     return 0;

}



void LoginTrapHandler(LONG trapParm)

{

     /*   This function gets called with the connection number of the

          user just logging in                                        */



     if((trapInPtr+1)%MAX_EVT==trapOutPtr)   // if queue full, ignore request...

          return;                       // this should be VERY unlikely!

     trapInPtr=(trapInPtr+1)%MAX_EVT;        // implement circular buffer

     trapConnection[trapInPtr]= (unsigned)trapParm; // fill in queue entry

     trapType[trapInPtr]= EVENT_LOGIN_USER;   // set signal type

     if(!threadState)                   // Restart main process if dormant

          ResumeThread(myThreadID);

}



void LogoutTrapHandler(LONG trapParm)

{

WORD   objectType;

long   objectID;

BYTE   loginTime[8];



     /*   This function gets called with the connection number of the

          user just logging out                                       */



     if((trapInPtr+1)%MAX_EVT==trapOutPtr)

          return;                       // ignore signal if queue is full

     trapInPtr=(trapInPtr+1)%MAX_EVT;

     trapConnection[trapInPtr]= (unsigned)trapParm;    // Place event in queue

     trapType[trapInPtr]= EVENT_LOGOUT_CONNECTION; //EVENT_CLEAR_CONNECTION;

     GetConnectionInformation((unsigned)trapParm,trapUser[trapInPtr],

     &objectType,&objectID,loginTime);

     if(!threadState)                   // restart main process if sleeping.

          ResumeThread(myThreadID);



}



void mountTrapHandler(LONG trapParm)

{

     /*   This function gets called with the number of the volume being

          mounted                                                     */



     if(trapParm==scratchVolumeNumber)

          scratchVolumeIsMounted=1;

}



void dismountTrapHandler(LONG trapParm)

{

     /*   This function gets called with the number of the volume being

          dismounted                                                  */



     if(trapParm==scratchVolumeNumber)

          scratchVolumeIsMounted=0;

}



void DownHandler()

{

     UnregisterForEvent(loginEventHandle);

     UnregisterForEvent(logoutEventHandle);

     UnregisterForEvent(mountEventHandle);

     UnregisterForEvent(dismountEventHandle);

     NWDSFreeContext(dContext);

}



/* check if good username, i.e. 3 characters and 5 digits as in abc12345

   returns 0 if ok 1 else

*/



int badName(char *uname)

{

int i;

for (i=0;i<3;i++)

  if (!isalpha(uname[i]))

    return 1;

for (i=3;i<8;i++)

  if (!isdigit(uname[i]))

    return 1;

return 0;

}



/* check if bad directory, i.e. too old or invalid owner */



int badDir(DIR *dp)

{

char *owner;

time_t t1;

SPECIAL *spp;



owner=myMapLocalObjectIDToName(LongSwap(dp->d_uid));

if (strnicmp(dp->d_name,owner+3,8))

  {

  if (debugFlag)

    printf("AUTODEL: %s Bad owner for %s (%s)\n",timeStr(),dp->d_name,owner);

  // if mapping of object id fails "??" is returned for name

  if (stricmp(owner,"??")) // owner was read correctly

    return 1;

  }

t1=_ConvertDOSTimeToCalendar(dp->d_cdatetime);

updateSpecial();

if (spp=bsearch(dp->d_name,special,specialCount,sizeof(SPECIAL),stricmp))



if ((time(NULL)-t1)/3600l > (spp ? spp->maxAge : maxAge))

  {

  if (debugFlag)

    printf("AUTODEL: %s %s older than %d hours\n",timeStr(),dp->d_name,

    (spp ? spp->maxAge : maxAge));

  return 1;

  }

return 0;

}



/* auto deletion thread



   Every time the thread is active it scans the base directory.

   For any directory :

     If name != owner name remove dir,

     if age of directory greater maxAge remove dir

*/



void autoDeleteThread(void *p)

{

time_t t;

DIR * dirPtr, *direntPtr;

char tempPath[256];



do {

   delay(30000);

   ThreadSwitch();

   t=time(NULL);

   sprintf(tempPath,"%s\\*.*",scratchDir);



   // First open the base directory for scanning; continue if it is empty



   if ((dirPtr=opendir(tempPath))==NULL)

     continue;

   while ( (direntPtr=readdir(dirPtr)) !=NULL)

     {

     sprintf(tempPath,"%s\\%s",scratchDir,direntPtr->d_name);

     if (direntPtr->d_attr & ATTRIB_DIR)    // Is entry a directory?

       {

       ThreadSwitch();

       if (badDir(direntPtr))

          {

          KillSubTree(tempPath,0);

          rmdir(tempPath);

          }

        }

     }

   closedir(dirPtr);                         // done scanning directory

   } while(1);

}



char *timeStr(void)

{

static char tmp[16];

time_t t;

struct tm tnow;

t=time(NULL);

localtime_r (&t,&tnow);



strftime(tmp,16,"%H:%M:%S",&tnow);

return tmp;

}





void main(int argc, char *argv[])

{

int spaceRestriction=0,retCode,useQuotas=0;

unsigned rightsMask=0,length,purgeFlag=0,cleanFlag=0;

unsigned counter,counter2,autoDeleteFlag=0;

char tempPath[256],objectName[256],volumeName[17],option,pathGivenFlag=0;

char directoryPath[256],*p1,*p2;

BYTE loginTime[10];

WORD connection,objectType;

long objectID;

SPECIAL sp,*spp;

// Set default rights mask

rightsMask=TA_READ|TA_WRITE|TA_DELETE|TA_SEEFILES|TA_MODIFY|TA_CREATE;



/* First get the command line args */



if (argc<2)

  {

  PrintHelp();

  return;

  }



puts("");



dContext = NWDSCreateContext( );

if (dContext==ERR_CONTEXT_CREATION)

  {

  ConsolePrintf("Error creating context\n");

  return;

  }



/* OK, make sure the base work path exists */



counter=1;

while(counter<argc)

  {

  // is argument currently pointed to a valid option -x or /x ?

  if( ((argv[counter][0]!='/')&&(argv[counter][0]!='-')) || (!argv[counter][1]) )

    {

    printf("Expected switch, but instead found \"%s\"\n",argv[counter]);

    PrintHelp();

    return;

    }

  option=argv[counter][1] & 0xdf;    // get switch letter into 'option'.

                                     // and change to upper case.

  length=0;

  if (counter<argc-1)       // if argument follows then set 'length' to

                           // its true length.

  if (argv[counter+1][0]!='-')

     length=strlen(argv[counter+1]);



  switch (option)

    {

    case 'Q':

      if(!length)

        {

        printf("Missing or bad value for option \"%s\"\n",argv[counter]);

        PrintHelp();

        return;

        }



      if (!isdigit(argv[counter+1][0]))

         {

         printf("Expected unsigned numeric space limit, but instead \nfound \"%s\".\n",argv[2]);

         PrintHelp();

         return;

         }



      useQuotas=1;

      p1=strrchr(argv[counter+1],0)-1;

      if (toupper(*p1)=='M') // mega bytes

        {

        *p1=0;

        spaceRestriction=(atol(argv[counter+1]))*256;

        }

      else

        {

        if (!isdigit(*p1))

          *p1=0;

        spaceRestriction=(atol(argv[counter+1])+3)/4;

        }

      // function expects units of 4096, not KB; round up.

      counter+=2;

      break;



   case 'C':

     cleanFlag=1;

     counter++;

     break;



   case 'V':

     debugFlag=1;

     counter++;

     break;



   case 'D':

     if (!length)

       {

       printf("Missing or bad value for option \"%s\"\n",argv[counter]);

       PrintHelp();

       return;

       }

     if (length>100)

       {

       puts("Working path given is too long.\n");

       return;

       }



     chdir("SYS:\\");

     strcpy(tempPath,argv[counter+1]);

     strcpy(scratchDir,strupr(tempPath));

     if (chdir(tempPath))      // make base directory current default

       {

       printf("\nBase directory \"%s\" is invalid\n",tempPath);

       return;

       }



     if (ParsePath(tempPath,objectName,volumeName,directoryPath))

       {

       printf("\nBase directory \"%s\" is invalid\n",tempPath);

       return;

       }



     if (GetVolumeNumber(volumeName,&scratchVolumeNumber))

       {

       printf("\nBase directory volume \"%s\" is invalid\n",volumeName);

       return;

       }



     counter+=2;

     pathGivenFlag=1;

     break;



   case 'P':

     purgeFlag=1;

     counter++;

     break;



   case 'A':

     autoDeleteFlag=1;

     if (!length)

       {

       printf("Missing or bad value for option \"%s\"\n",argv[counter]);

       PrintHelp();

       return;

       }

     p2=argv[counter+1];

     if ((maxAge=atoi(p2))==0)

       {

       printf("Invalid value (%s) for delete interval\n",p2);

       return;

       }

     counter+=2;

     break;



   case 'T':

     if (!length)

       {

       printf("Missing or bad value for option \"%s\"\n",argv[counter]);

       PrintHelp();

       return;

       }

     rightsMask=0;            // Clear out default value

     for(counter2=0;counter2<length;counter2++)

       {

       // look at each letter in rights list.

       switch(argv[counter+1][counter2] & 0xdf)

       {

       case 'R':

         rightsMask|=TA_READ;

         break;

       case 'W':

         rightsMask|=TA_WRITE;

         break;

       case 'C':

         rightsMask|=TA_CREATE;

         break;

       case 'E':

         rightsMask|=TA_DELETE;

         break;

       case 'A':

         rightsMask|=TA_ACCESSCONTROL;

         break;

       case 'F':

         rightsMask|=TA_SEEFILES;

         break;

       case 'M':

         rightsMask|=TA_MODIFY;

         break;

       case 'S':

         rightsMask|=TA_SUPERVISOR;

         break;

       default:

         printf("Improper trustee right provided: \"%c\"\n",argv[3][counter]);

         PrintHelp();

         return;

         }

      }

      counter+=2;

      break;

    default:

      printf("Bad option - \"%s\"\n",argv[counter]);

      PrintHelp();

      return;

    }

  }



if (!pathGivenFlag)

  {

  puts("A base path MUST be specified with -D base_path");

  PrintHelp();

  return;

  }



trapInPtr=trapOutPtr=0;       // Zero the pending trap queue pointers

myThreadID=GetThreadID();     // Get this NLM's thread ID.

threadState=1;                // Set my status as running (see trap functions)



//   Activate the traps!



loginEventHandle=RegisterForEvent(EVENT_LOGIN_USER,LoginTrapHandler,NULL);

//     logoutEventHandle=RegisterForEvent(EVENT_CLEAR_CONNECTION,LogoutTrapHandler,NULL);

logoutEventHandle=RegisterForEvent(EVENT_LOGOUT_CONNECTION,LogoutTrapHandler,NULL);

mountEventHandle=RegisterForEvent(EVENT_ANY_VOL_MOUNT,mountTrapHandler,NULL);

dismountEventHandle=RegisterForEvent(EVENT_ANY_VOL_DISMOUNT,dismountTrapHandler,NULL);



atexit(DownHandler);



// Do not attempt to operate on dismounted volume! Check first!



if (GetVolumeInformation(0,scratchVolumeNumber,sizeof(VOLUME_STATS),&scratchVolumeStatistics))

  {

  printf("** Warning - scratch volume (%s) is not mounted!\n",volumeName);

  }

else

  if (!scratchVolumeStatistics.isMounted)

    {

    printf("** Warning - scratch volume (%s) is not mounted!\n",volumeName);

    }

  else

    scratchVolumeIsMounted=1;



if (autoDeleteFlag) {

  if (BeginThread(autoDeleteThread,NULL,8192,NULL)==EFAILURE)

    {

    printf("Error creating auto delete thread\n");

    }

  printf("Maximum age of directories is %d hours.\n",maxAge);

  }



if (useQuotas)

  printf("Quotas set to %ld KB\n",spaceRestriction<<2);

if (debugFlag)

  printf("Verbose mode\n");



for(;;)

  {

  while(trapInPtr==trapOutPtr)       // If no events in queue, go to sleep.

    {

    threadState=0;                // trap handlers wake the thread if

    SuspendThread(myThreadID);    // anything happens.

    }



  threadState=1;                          // If traps happen now, let

                                          // them know we're already awake.

  trapOutPtr=(trapOutPtr+1)%MAX_EVT;           // advance pointer

  connection=trapConnection[trapOutPtr];  // get connection of this event

  if (connection<1)

     continue;                          // Ignore log in or out

                                        // of NLM service connections.



  sprintf(tempPath,"%d",connection);      // get number as string.



  if (!scratchVolumeIsMounted)

    {

    if (debugFlag)

      printf("** Warning: scratch volume %s is not available; operation ignored\n",volumeName);

    continue;

    }



  switch(trapType[trapOutPtr])            // Service the queued trap.

    {

    case EVENT_LOGIN_USER:

      GetConnectionInformation(connection,objectName,&objectType,&objectID,loginTime);

      if (badName(objectName))

        break;

      sprintf(tempPath,"%s\\%.8s",scratchDir,objectName);

      if (debugFlag)

        printf("LOGIN:   %s %s (%d)\n",timeStr(),objectName,connection);

      if (updateSpecial())

        if (debugFlag)

          printf("UPDATE:  %s %s updated\n",timeStr(),SPECIAL_FILE);

      sprintf(sp.owner,"%.8s",objectName);

      if (spp=bsearch(&sp,special,specialCount,sizeof(SPECIAL),stricmp))

        retCode=MakeDirectory(tempPath,objectID,rightsMask,spp->quota,purgeFlag);

      else

        retCode=MakeDirectory(tempPath,objectID,rightsMask,spaceRestriction,purgeFlag);

      break;



//               case EVENT_CLEAR_CONNECTION:

    case EVENT_LOGOUT_CONNECTION:

//                    GetConnectionInformation(connection,objectName,&objectType,&objectID,loginTime);

      strcpy(objectName,trapUser[trapOutPtr]);

      sprintf(tempPath,"%.8s",objectName);

      if (badName(objectName))

        break;

      if (debugFlag)

        printf("LOGOUT:  %s %s\n",timeStr(),objectName);

      FlagEntryAsNPurge(tempPath,purgeFlag);  // Remove any special

                                              // attribs.

      if (cleanFlag)

        if (emptyDir(tempPath))

          {

          if (debugFlag)

            printf("LOGOUT:  %s Removing empty temp directory %s\n",timeStr(),tempPath);

//    KillSubTree(tempPath,0);

          rmdir(tempPath);

          }

      retCode=ZapDirectoryTrustees(tempPath);

      break;

    }

  }

}

