/*

     GROUPD.C - Add user to group if he or she logs in from a station

     whose node address is in a list of allowed addresses. If the node

     is not in the list, the user is removed from the group.



     Copyright (c) 1997 Peter Heitzer

     peter.heitzer@rz.uni-regensburg.de

     Idea by Josef Spangler.



     Some parts taken from GRANTD.C

     (c) 1993 Ronald W. Butts

     The Wichita State University

     Wichita, KS 67260-0044



*/



#include <ctype.h>

#include <stdlib.h>

#include <stdio.h>

#include <stddef.h>

#include <stdarg.h>

#include <fcntl.h>

#include <conio.h>

#include <direct.h>

#include <nwfile.h>

#include <nwdir.h>

#include <string.h>

#include <advanced.h>

#include <process.h>

#include <niterror.h>

#include <errno.h>

#include <nwconn.h>

#include <nwdir.h>

#include <io.h>

#include <nwenvrn.h>

#include <nwmisc.h>

#include <time.h>

#include <nwsnut.h>

#include <nwdsbuft.h>

#include <nwdsapi.h>

#include <nwcntask.h>



#define MAX_EVT 50

#define PLINES 8

#define CONFIG "SYS:SYSTEM\\GROUPD.CFG"

#define LOGFILE "SYS:SYSTEM\\GROUPD.LOG"

#define OLDCONFIG "SYS:SYSTEM\\GROUPD.OLD"

#define TMPCONFIG "SYS:SYSTEM\\GROUPD.TMP"

#define MAX_NODE 1024

#define MIN_PER_DAY 1440



typedef BYTE A_NETWORK[4];

typedef BYTE A_NODE[6];



typedef struct {

        A_NETWORK network;

        A_NODE node;

        } NET_ADDR;



typedef struct netnode {

        NET_ADDR addr;

        char name[16];

        } NODE;



typedef struct {

        int hours;

        int minutes;

        } INTERVAL;



struct  {

       char userName[80];            // user name the NLM logs in with

       char groupName[80];           // group to manage

       char encryptedPassWord[128];

       char cleanupTime[32];

       } gParam;



enum { O_USER, O_GROUP, O_PASS, O_CLEAN, O_NONE  };



typedef struct {

               char *optname;

               int opt;

               } OPTION;



int removeGroupMembers(void);

int removeGroupMembers0(void);

int updateConfiguration(void);

int updateStatus(void);

int userActive(char *uname);



typedef struct {

        time_t startTime;         // in minutes from 00:00

        time_t interval;          // interval in minutes between activation

        time_t lastTime;

        int (*cronfunc)(void) ;   // function to be activated

        } CRONSTRUCT;



enum  { CR_REMOVE, CR_UPDATE, CR_SMART };



CRONSTRUCT cronTab[] = {

        { 0,86400l, 0,removeGroupMembers0 },

        { 0,60,0, updateConfiguration },

        { 0,300,0, updateStatus }

        };



struct {

       INTERVAL activationTime;

       int done;

       } crontab;



enum { E_OK, E_OPEN, E_NO_NODES, E_NO_STARTUP, E_READ };



OPTION options[]={

  {"User",O_USER },

  {"Group",O_GROUP },

  {"Password",O_PASS},

  {"CleanUpTime",O_CLEAN},

  {NULL, O_NONE},

  };



unsigned trapInPtr,trapOutPtr,threadState;

unsigned trapConnection[MAX_EVT],trapType[MAX_EVT];

int      clearRequest=0;



char *timeStr(void);

char *dateStr(void);

NODE *nodes,*newNodes;



// NDS variables

NWDSContextHandle   dContext;

pBuf_T readBuf, aBuf; // buffer for NDSRead

pBuf_T netaBuf, aBuf2;



// special nodes

NODE wildNet={"\xff\xff\xff\xff","","\0"},

     wildNode={"","\xff\xff\xff\xff\xff\xff","\0"},

     serverNode={"","\x00\x00\x00\x00\x00\x01","\0"};



int  newNodeCount=0,nodeCount=0,memberCount=0;

LONG loginEventHandle,logoutEventHandle;

int myThreadID;



// for NUT stuff

NUTInfo *handle;

LONG    NLMHandle;

LONG    allocTag;

int     CLIBScreenID,escapeMode,inForm=0,pLine=0;

PCB     *myPortal,*statusPortal;

LONG    infoPortalNumber,statusPortalNumber;

PCB     *ipPtr;

WORD    screenHeight, screenWidth;

long    addCount=0, remCount=0;



enum { MSG_NAME, MSG_VER, MSG_LIST, MSG_UPDATE, MSG_STATUS, MSG_CLEAR, MSG_ENCRYPT };

char    *messages[16]={ "GROUPD",

                        "1.06 (c) 1997 Peter Heitzer",

                        "GROUPD control",

                        "Update configuration",

                        "Update status display",

                        "Clear trustee group",

                        "Write new password to config",

                        "Exit" };



#define SECT_STARTUP "[Startup]"

#define SECT_NODES "[Nodes]"



// log error messages

void logPrintf(char *format, ... )

{

FILE *f;

static char msgbuf[256];



va_list arglist;

va_start (arglist, format);

sprintf(msgbuf,"%s %s: ",dateStr(),timeStr());

vsprintf (strrchr(msgbuf,0), format, arglist);

va_end (arglist);

if (f=fopen(LOGFILE,"a"))

  {

  fprintf(f,msgbuf);

  fclose(f);

  }

}



void portalPrint(char *s)

{

if (pLine<PLINES)

  {

  NWSDisplayTextInPortal(pLine,0,s,VNORMAL,myPortal);

  pLine++;

  }

else

  {

  NWSScrollPortalZone(0,0,PLINES,78,0,1,V_UP,myPortal);

  NWSDisplayTextInPortal(PLINES-1,0,s,VNORMAL,myPortal);

  }

NWSUpdatePortal(myPortal);

}



void statusPrint(char *s)

{

NWSClearPortal(statusPortal);

NWSDisplayTextInPortal(0,0,s,VNORMAL,statusPortal);

NWSUpdatePortal(statusPortal);

}



// start clearThread

int removeGroupMembers0(void)

{

clearRequest=1;

return 0;

}



// display node

char *nodestr(NODE *n,int nameFlag)

{

static char buf[16];

if (nameFlag && n->name[0])

  sprintf(buf,"%-12s",n->name);

else

  sprintf(buf,"%02X%02X%02X%02X%02X%02X",n->addr.node[0],n->addr.node[1],n->addr.node[2],

  n->addr.node[3],n->addr.node[4],n->addr.node[5]);

return buf;

}



// compare two nodes

int nodecmp(const void *node1, const void *node2)

{

return memcmp(node1,node2,sizeof(NET_ADDR));

}



// compare two network addresses

int nodenetcmp(const NODE *node1, const NODE *node2)

{

return memcmp(node1->addr.network,node2->addr.network,sizeof(A_NETWORK));

}



// compare two network addresses

int nodenodecmp(const NODE *node1, const NODE *node2)

{

return memcmp(node1->addr.node,node2->addr.node,sizeof(A_NODE));

}



NODE *nodeOK(NODE *n)

{

NODE *n1;

if (n1=bsearch(n,nodes,nodeCount,sizeof(NODE),nodecmp))

  return n1;

memcpy(wildNet.addr.node,n->addr.node,sizeof(wildNet.addr.node));

if (n1=bsearch(&wildNet,nodes,nodeCount,sizeof(NODE),nodecmp))

  return n1;

memcpy(wildNode.addr.network,n->addr.network,sizeof(wildNode.addr.network));

if (n1=bsearch(&wildNode,nodes,nodeCount,sizeof(NODE),nodecmp))

  return n;

return NULL;

}



/* encrypt password

   the algorithm is simple:

   The sum of the characters in the key (converted to uppercase first)

   is taken as a seed for srand()

   The password is bytewise xored with rand() and each byte stored as a hex

   string, e.g. 4A5EBA ...

*/



char *encrypt(char *password, char *key)

{

static char ebuf[256];

int i,seed;



for (i=0,seed=0;i<strlen(key);i++)

  seed+=toupper(key[i]);

srand(seed);

for (i=0;i<strlen(password);i++)

  sprintf(ebuf+i*2,"%02X",password[i]^(rand() %256));

return ebuf;

}



/*

decrypt password encrypted by the algorithm in encrypt();

*/



char *decrypt(char *password, char *key)

{

static char ebuf[256];

char tmp[3];

int i,seed;



for (i=0,seed=0;i<strlen(key);i++)

  seed+=toupper(key[i]);

srand(seed);



for (i=0;i<strlen(password);i+=2)

  {

  sprintf(tmp,"%.2s",password+i);

  sprintf(ebuf+i/2,"%c",(strtoul(tmp,NULL,16)& 0xff )^(rand() %256));

  }

return ebuf;

}



int x2b(char *hexstr, BYTE *binstr)

{

char tmp[3],*p;

int i;



for (i=0,tmp[2]=0;i<strlen(hexstr)/2;i++)

  {

  strncpy(tmp,hexstr+i*2,2);

  binstr[i]=(BYTE) strtoul(tmp,&p,16);

  if (*p)

    return 1;

  }

return 0;

}



// read configuration file. If updateFlag is true, only read node information



int readConfigFile(int updateFlag)

{

FILE *f;

int opt,state=0,i;

char *line,buf[80],*p1,*p2,*p3;

NODE node;

if ((f=fopen(CONFIG,"r"))==NULL)

  return E_OPEN;



newNodeCount=0;



do {

   if (fgets(buf,80,f)==NULL)

     break;

   line=strtok(buf,"\n\r");

   if (line==NULL)

     continue;

   if (*line==';')

     continue;

   if (!stricmp(line,SECT_STARTUP))

     {

     state=1;

     continue;

     }

   if (!stricmp(line,SECT_NODES))

     {

     state=2;

     continue;

     }

   switch (state) {

     // read startup information

     case 1:

       if (updateFlag)

         break;

       p1=p2=NULL;

       if (p1=strtok(line,"="))

         p2=strtok(NULL,"");

       if (p2==NULL)

         break;

       for (i=0,opt=O_NONE;options[i].optname;i++)

         {

         if (!stricmp(p1,options[i].optname))

           {

           opt=options[i].opt;

           break;

           }

         }

       switch (opt) {

         case O_USER:

           strcpy(gParam.userName,p2);

           break;

         case O_GROUP:

           strcpy(gParam.groupName,p2);

           break;

         case O_PASS:

           strcpy(gParam.encryptedPassWord,p2);

           break;

         case O_CLEAN:

           strcpy(gParam.cleanupTime,p2);

         }

     // read nodes

     case 2:

       p1=p2=p3=NULL;

       if (newNodeCount>MAX_NODE)

         break;

       if (p1=strtok(line,","))

         if (p2=strtok(NULL,","))

           p3=strtok(NULL,"");

       if (p2==NULL)

         break;

       /* if (p3)

         break; */

       if (!strcmp(p1,p2))

         break;

       memset(&node,0,sizeof(NODE));

       if (!strcmp(p1,"*"))

         memset(node.addr.network,0xff,sizeof(A_NETWORK));

       else

         {

         if (strlen(p1)!=8)

           break;

         if (x2b(p1,node.addr.network))

           break;

         }

       if (!strcmp(p2,"*"))

         memset(node.addr.node,0xff,sizeof(A_NODE));

       else

         {

         if (strlen(p2)!=12)

           break;

         if (x2b(p2,node.addr.node))

           break;

         }

       if (p3) {

         *(p3+15)=0;

         strcpy(node.name,p3);

         }

       memcpy(&newNodes[newNodeCount++],&node,sizeof(NODE));

     }

   } while(TRUE);



fclose(f);

if (newNodeCount)

  return E_OK;

else

  return E_NO_NODES;

}



int updateConfiguration(void)

{

char tmp[64];

NODE *ntmp;

if (readConfigFile(TRUE))

  return -1;

qsort(newNodes,newNodeCount,sizeof(NODE),nodecmp);

ntmp=nodes; nodes=newNodes; newNodes=ntmp;  // swap node arrays

if (nodeCount!=newNodeCount)

  {

  sprintf(tmp,"%s New node count: %d",timeStr(),newNodeCount);

  portalPrint(tmp);

  }

nodeCount=newNodeCount;

// portalPrint(timeStr());

return 0;

}



/* write encrypted password to configuration file */



void writeEncryptedPassWordToConfiguration(void)

{

FILE *f0,*f1;

char buf[80];

if ((f0=fopen(CONFIG,"r"))==NULL)

  return;

if ((f1=fopen(TMPCONFIG,"w"))==NULL)

  {

  fclose(f0);

  return;

  }

fprintf(f1,"%s\n",SECT_STARTUP);

fprintf(f1,"%s=%s\n",options[O_GROUP].optname,gParam.groupName);

fprintf(f1,"%s=%s\n",options[O_USER].optname,gParam.userName);

fprintf(f1,"%s=%s\n",options[O_PASS].optname,gParam.encryptedPassWord);

fprintf(f1,"%s=%s\n\n",options[O_CLEAN].optname,gParam.cleanupTime);

do {

   if (fgets(buf,80,f0)==NULL)

     break;

   if (!strnicmp(buf,SECT_NODES,strlen(SECT_NODES)))

     break;

   } while(!feof(f1));



fprintf(f1,"%s",buf);

do {

   if (fgets(buf,80,f0)==NULL)

     break;

   fprintf(f1,"%s",buf);

   } while (!feof(f0));

fclose(f1); fclose(f0);

remove(OLDCONFIG);

rename(CONFIG,OLDCONFIG);

rename(TMPCONFIG,CONFIG);

}



/* add/remove group to/from user's attributes "Security Equals" and "Group Membership" */



int UpdateUser(char *username,char *group,int absFlag,nuint32 changeType)

{

int ccode;

char tmp[256],User[256];

pBuf_T modBuf;



if (*username!='.' && absFlag)

  sprintf(tmp,".%s",username);

else

  strcpy(tmp,username);



*User='.';

NWDSCanonicalizeName(dContext,tmp,User+1);

if (ccode=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &modBuf))

  {

  logPrintf("NWDSAllocBuf: %x\n",ccode);

  return ccode;

  }



do {

  if (ccode = NWDSInitBuf(dContext, DSV_MODIFY_ENTRY, modBuf))

    {

    logPrintf("NWDSInitBuf: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutChange(dContext,modBuf,changeType,"Security Equals"))

    {

    logPrintf("NWDSPutChange: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutAttrVal(dContext,modBuf,SYN_DIST_NAME,group))

    {

    logPrintf("NWDSPutAttrVal: (%s) %x\n",User,ccode);

    break;

    }



  if (ccode = NWDSModifyObject(dContext,(pnstr8)User,NULL,(nbool8)0,modBuf))

    {

    if (ccode==((changeType==DS_ADD_VALUE) ? ERR_DUPLICATE_VALUE : ERR_NO_SUCH_VALUE))

      ccode=0;

    else

      logPrintf("line %ld NWDSModifyObject: %s %s %x\n",__LINE__,User,group,ccode);

    }



  if (ccode = NWDSInitBuf(dContext, DSV_MODIFY_ENTRY, modBuf))

    {

    logPrintf("NWDSInitBuf: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutChange(dContext,modBuf,changeType,"Group Membership"))

    {

    logPrintf("NWDSPutChange: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutAttrVal(dContext,modBuf,SYN_DIST_NAME,group))

    {

    logPrintf("NWDSPutAttrVal: (%s) %x\n",User,ccode);

    break;

    }



  if (ccode = NWDSModifyObject(dContext,(pnstr8)User,NULL,(nbool8)0,modBuf))

    {

    if (ccode==((changeType==DS_ADD_VALUE) ? ERR_DUPLICATE_VALUE : ERR_NO_SUCH_VALUE))

      ccode=0;

    else

      logPrintf("NWDSModifyObject: %s %s %x\n",User,group,ccode);

    }

  } while (FALSE);



NWDSFreeBuf(modBuf);

return ccode;

}



/* add/remove user to/from groups's attributes "Member" and "Equivalent to Me" */



int UpdateGroup(char *username,char *group,int absFlag,nuint32 changeType)

{

int ccode;

char tmp[256],User[256];

pBuf_T modBuf;



if (*username!='.' && absFlag)

  sprintf(tmp,".%s",username);

else

  strcpy(tmp,username);

*User='.';

NWDSCanonicalizeName(dContext,tmp,User+1);

if (ccode=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &modBuf))

  {

  logPrintf("NWDSAllocBuf: %x\n",ccode);

  return ccode;

  }



do {

  if (ccode = NWDSInitBuf(dContext, DSV_MODIFY_ENTRY, modBuf))

    {

    logPrintf("NWDSInitBuf: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutChange(dContext,modBuf,changeType,"Member"))

    {

    logPrintf("NWDSPutChange: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutAttrVal(dContext,modBuf,SYN_DIST_NAME,User))

    {

    logPrintf("NWDSPutAttrVal: (%s) %x\n",group,ccode);

    break;

    }



  if (ccode = NWDSModifyObject(dContext,(pnstr8)group,NULL,(nbool8)0,modBuf))

    {

    if (ccode==((changeType==DS_ADD_VALUE) ? ERR_DUPLICATE_VALUE : ERR_NO_SUCH_VALUE))

      ccode=0;

    else

      logPrintf("UG modify: %s, %s ->%x\n",group,User,ccode);

    }



  if (ccode = NWDSInitBuf(dContext, DSV_MODIFY_ENTRY, modBuf))

    {

    logPrintf("NWDSInitBuf: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutChange(dContext,modBuf,changeType,"Equivalent To Me"))

    {

    logPrintf("NWDSPutChange: %x\n",ccode);

    break;

    }



  if (ccode = NWDSPutAttrVal(dContext,modBuf,SYN_DIST_NAME,User))

    {

    logPrintf("NWDSPutAttrVal: (%s) %x\n",group,ccode);

    break;

    }



  if (ccode = NWDSModifyObject(dContext,(pnstr8)group,NULL,(nbool8)0,modBuf))

    {

    if (ccode==((changeType==DS_ADD_VALUE) ? ERR_DUPLICATE_VALUE : ERR_NO_SUCH_VALUE))

      ccode=0;

    else

      logPrintf("NWDSModifyObject: %s %s %x\n",User,group,ccode);

    }

  } while (FALSE);

NWDSFreeBuf(modBuf);

return ccode;

}



/* check if user is logged in, i.e. has values in "Network Address"

   returns 0 number of values or NDS error code */



int userActive(char *uname)

{

NWDSCCODE           ccode;

nuint32             syntax;

nint32              iterHandle=-1L;

nuint32             totalAttrs, totalValues;

char                attrName[256];

int                 el;



if (ccode = NWDSInitBuf(dContext, DSV_READ, aBuf2))

  return ccode;



if (ccode=NWDSGetSyntaxID(dContext,"Network Address",&syntax))

  return ccode;



if (ccode = NWDSPutAttrName(dContext,aBuf2,"Network Address"))

  return ccode;



totalValues=42; // set to value >0

do {

   if (ccode = NWDSRead(dContext,(pnstr8) uname,DS_ATTRIBUTE_VALUES,

               FALSE,aBuf2,&iterHandle,netaBuf))

               { el=__LINE__; break; }

   if (ccode=NWDSGetAttrCount(dContext,netaBuf,&totalAttrs))

     { el=__LINE__; break; }

   if (totalAttrs==0)

     return 0;

   if (ccode=NWDSGetAttrName(dContext,netaBuf,attrName,&totalValues,&syntax))

     { el=__LINE__; break; }

   } while (FALSE);

NWDSCloseIteration(dContext,iterHandle,DSV_READ);

/* sprintf(attrName,"%s: %d line: %d %x",uname,totalValues,el,ccode);

portalPrint(attrName); */

if (ccode==ERR_NO_SUCH_ATTRIBUTE) // no network addresses found

  return 0;

return totalValues;

}



/* remove inactive user */



void removeInactiveUser(char *uname)

{

/*

if (!userActive(uname))

  {

  UpdateUser(uname,gParam.groupName,0,DS_REMOVE_VALUE);

  UpdateGroup(uname,gParam.groupName,0,DS_REMOVE_VALUE);

  }

else

*/

  memberCount++;

}



/* remove all members of trustee group */



int removeGroupMembers(void)

{

NWDSCCODE           ccode;

nuint32             syntax;

char                *p;

nint32              iterHandle=-1L;

nuint32             totalAttrs, attrValSize, totalValues, x;

char                attrValue[1024],memberName[256],attrName[256];

time_t              t;



#define aName "Member"



if (ccode = NWDSInitBuf(dContext, DSV_READ, aBuf))

  {

  return ccode;

  }



if (ccode=NWDSGetSyntaxID(dContext,aName,&syntax))

  {

  return ccode;

  }



if (ccode = NWDSPutAttrName(dContext,aBuf,aName))

  {

  return ccode;

  }



t=time(NULL);



do {

   if (ccode = NWDSRead(dContext,(pnstr8) gParam.groupName,DS_ATTRIBUTE_VALUES,

               FALSE,aBuf,&iterHandle,readBuf))

               break;

   if (NWDSGetAttrCount(dContext,readBuf,&totalAttrs))

     break;

   if (totalAttrs==0)

     break;

   if (NWDSGetAttrName(dContext,readBuf,attrName,&totalValues,&syntax))

     break;

   for(x=totalValues;x;x--)

     {

     if (NWDSComputeAttrValSize(dContext,readBuf,syntax,&attrValSize))

       break;

     if (NWDSGetAttrVal(dContext,readBuf,syntax,attrValue))

       break;

/*     if (strnicmp(attrValue,"CN=",3))

       sprintf(memberName,"CN=%s",attrValue);

     else */

       strcpy(memberName,attrValue);

//    portalPrint(attrValue);

     UpdateUser(memberName,gParam.groupName,0,DS_REMOVE_VALUE);

     UpdateGroup(memberName,gParam.groupName,0,DS_REMOVE_VALUE);

     }

   } while (iterHandle != NO_MORE_ITERATIONS);

sprintf(attrName,"%s Group cleared in %ld s",timeStr(),time(NULL)-t);

portalPrint(attrName);

return 0;

}



/* scan members of trustee group and optionally invoke function */



int scanGroupMembers(void (*memberfunc(char *uname)))

{

NWDSCCODE           ccode;

nuint32             syntax;

char                *p;

nint32              iterHandle=-1L;

nuint32             totalAttrs, attrValSize, totalValues, x;

char                attrValue[1024],memberName[256],attrName[256];

#define aName "Member"



if (ccode = NWDSInitBuf(dContext, DSV_READ, aBuf))

  {

  return ccode;

  }



if (ccode=NWDSGetSyntaxID(dContext,aName,&syntax))

  {

  return ccode;

  }



if (ccode = NWDSPutAttrName(dContext,aBuf,aName))

  {

  return ccode;

  }



do {

   if (ccode = NWDSRead(dContext,(pnstr8) gParam.groupName,DS_ATTRIBUTE_VALUES,

               FALSE,aBuf,&iterHandle,readBuf))

               break;

   if (NWDSGetAttrCount(dContext,readBuf,&totalAttrs))

     break;

   if (totalAttrs==0)

     break;

   if (NWDSGetAttrName(dContext,readBuf,attrName,&totalValues,&syntax))

     break;

   for(x=totalValues;x;x--)

     {

     if (NWDSComputeAttrValSize(dContext,readBuf,syntax,&attrValSize))

       break;

     if (NWDSGetAttrVal(dContext,readBuf,syntax,attrValue))

       break;

     if (strnicmp(attrValue,"CN=",3))

       sprintf(memberName,"CN=%s",attrValue);

     else

       strcpy(memberName,attrValue);

     if (memberfunc)

       memberfunc(memberName);

     // portalPrint(memberName);

     }

   } while (iterHandle != NO_MORE_ITERATIONS);

return 0;

}



/* add user to group if login from allowed node or remove from group else */



int AddUserToGroup(char *username,WORD connection)

{

NODE unode,*n;

char tmp[256];

GetInternetAddress(connection,unode.addr.network,unode.addr.node);

unode.name[0]=0;



/* if login from server, i.e. node=1 do nothing */

if (!memcmp(unode.addr.node,serverNode.addr.node,sizeof(A_NODE)))

  {

  sprintf(tmp,"%s Server login for %s ignored",timeStr(),username);

  portalPrint(tmp);

  return 0;

  }



if (n=nodeOK(&unode))

  {

  sprintf(tmp,"%s %s: Add %s -> %x %x\n",timeStr(),nodestr(n,1),username,

  UpdateGroup(username,gParam.groupName,1,DS_ADD_VALUE),

  UpdateUser(username,gParam.groupName,1,DS_ADD_VALUE));

  addCount++;

  }

else

  {

  sprintf(tmp,"%s %s: Remove %s -> %x %x\n",timeStr(),nodestr(&unode,0),username,

  UpdateGroup(username,gParam.groupName,1,DS_REMOVE_VALUE),

  UpdateUser(username,gParam.groupName,1,DS_REMOVE_VALUE));

  remCount++;

  }

portalPrint(tmp);

return 0;

}



void LoginTrapHandler(LONG trapParm)

{



     /*   This function gets called with the connection number of the

          user just logging in                                        */



     if((trapInPtr+1)%MAX_EVT==trapOutPtr)   // if queue full, ignore request...

          return;                       // this should be VERY unlikely!

     trapInPtr=(trapInPtr+1)%MAX_EVT;        // implement circular buffer

     trapConnection[trapInPtr]= (unsigned)trapParm; // fill in queue entry

     trapType[trapInPtr]= EVENT_LOGIN_USER;   // set signal type

     if(!threadState)                   // Restart main process if dormant

          ResumeThread(myThreadID);

}



/* Every time a user logs out or the connection is cleared by a watchdog

   timeout this handler is called.

   If the associated user has a "Network Address" property with only one

   entry then the user is removed from the trustee group thus keeping the

   group small */



void LogoutTrapHandler(LONG trapParm)

{

WORD   objectType;

long   objectID;

BYTE   loginTime[8];

char   username[256],*uname,tmp[256];

int    n;



return;

uname=username+4;

*uname=0;

GetConnectionInformation((unsigned)trapParm,uname,&objectType,&objectID,loginTime);

if (*uname)

  {

  if (strnicmp(uname,"CN=",3))

    {

    strncpy(username,".CN=",4);

    uname=username;

    }

  n=userActive(uname);

  sprintf(tmp,"userActive(%s): %d\n",uname,n);

  portalPrint(tmp);

  if (n==0 || n==1)

    {

    UpdateUser(uname,gParam.groupName,0,DS_REMOVE_VALUE);

    UpdateGroup(uname,gParam.groupName,0,DS_REMOVE_VALUE);

    portalPrint("Logout remove"); portalPrint(uname);

    }

  }

}



/* unload/exit cleanup function */



void cleanup()

{

if (loginEventHandle)

  UnregisterForEvent(loginEventHandle);

if (logoutEventHandle)

  UnregisterForEvent(logoutEventHandle);

if (newNodes)

   free(newNodes);

if (nodes)

  free(nodes);

if (handle)

  {

  NWSDestroyList (handle);

  NWSDestroyPortal (infoPortalNumber, handle);

  NWSDestroyPortal (statusPortalNumber, handle);

  NWSRestoreNut(handle);

  }

if (CLIBScreenID)

  DestroyScreen(CLIBScreenID);

if (dContext) {

  NWDSLogout(dContext);

  NWDSFreeContext(dContext);

  }

if (readBuf)

  NWDSFreeBuf(readBuf);

if (aBuf)

  NWDSFreeBuf(aBuf);

if (aBuf2)

  NWDSFreeBuf(aBuf2);

if (netaBuf)

  NWDSFreeBuf(netaBuf);

}



/*

   check if "good" username, i.e. 3 characters and 5 digits as in abc12345

   returns 0 if ok 1 else

*/



int badName(char *uname)

{

int i;

for (i=0;i<3;i++)

  if (!isalpha(uname[i]))

    return 1;

for (i=3;i<8;i++)

  if (!isdigit(uname[i]))

    return 1;

return 0;

}



/* convert string to time or interval */



int makeInterval(char *i,      /* string given as HHMM */

                 INTERVAL *ip,

                 int isTime    /* if true check if HH=0..23 and MM=0..59 */

               )

{

int t;

char *p;



if (strlen(i)!=4)

  return -1;

t=strtol(i,&p,10);

if (*p)

  return -1; // convertition failed

ip->hours=t / 100; ip->minutes=t % 100;



if (!isTime)

  return 0;



if (ip->hours >23)

  return -1;

if ( ip->minutes >59)

  return -1;

return 0;

}



int ListAction (LONG keyPressed, LIST **elementSelected, LONG *itemNumber, void *listParameter)

{

elementSelected = elementSelected;

listParameter = listParameter;



if (keyPressed == M_ESCAPE)

  return 1;



NWSPushList(handle);

NWSInitForm(handle);



switch (*itemNumber+MSG_UPDATE) {

  case MSG_UPDATE: if (NWSConfirm(MSG_UPDATE,10,40,0,NULL,handle,NULL)==1)

          updateConfiguration();

          break;

  case MSG_STATUS:

          updateStatus();

          break;

  case MSG_CLEAR: if (NWSConfirm(MSG_CLEAR,10,40,0,NULL,handle,NULL)==1)

            {

            // NWSSelectPortal(infoPortalNumber,handle);

            removeGroupMembers0();

            }

          break;

  case MSG_ENCRYPT: if (NWSConfirm(MSG_ENCRYPT,10,40,0,NULL,handle,NULL)==1)

          writeEncryptedPassWordToConfiguration();

          break;

  }

NWSPopList(handle);

NWSUpdatePortal(myPortal);

return -1;

}



int initNUT(void)

{



NLMHandle=GetNLMHandle();



CLIBScreenID = CreateScreen("GROUPD Screen", AUTO_DESTROY_SCREEN);

if (!CLIBScreenID)

  return -1;

allocTag = AllocateResourceTag(NLMHandle, "GROUPD Alloc Tag",AllocSignature);



if (!allocTag)

  {

  DestroyScreen(CLIBScreenID);

  return -1;

  }



if (NWSInitializeNut(MSG_NAME,MSG_VER,NORMAL_HEADER,NUT_REVISION_LEVEL,

    messages, 0, CLIBScreenID, allocTag, &handle))

        {

        DestroyScreen(CLIBScreenID);

        return -1;

        }



DisplayScreen(CLIBScreenID);

GetSizeOfScreen (&screenHeight, &screenWidth);



infoPortalNumber = NWSCreatePortal(14l,0l,PLINES+2,(LONG)screenWidth,

                        9l,(LONG)screenWidth-2, SAVE,

                        NULL,

                        VNORMAL, SINGLE, VNORMAL, CURSOR_OFF, VIRTUAL, handle);



if (infoPortalNumber > MAXPORTALS)

  {

  printf ("Unable to create information portal\n");

  DestroyScreen(CLIBScreenID);

  return -1;

  }



statusPortalNumber = NWSCreatePortal(11l,0l,3,(LONG)screenWidth,

                        3l,(LONG)screenWidth-2, SAVE,

                        NULL,

                        VNORMAL, SINGLE, VNORMAL, CURSOR_OFF, VIRTUAL, handle);



if (statusPortalNumber > MAXPORTALS)

  {

  printf ("Unable to create status portal\n");

  DestroyScreen(CLIBScreenID);

  return -1;

  }

return 0;

}



/* NUT handling thread */



void nutThread(void *p)

{

int i;

NWSGetPCB (&ipPtr, infoPortalNumber, handle);

myPortal=ipPtr;

NWSGetPCB (&ipPtr, statusPortalNumber, handle);

statusPortal=ipPtr;

NWSClearPortal(myPortal);

NWSClearPortal(statusPortal);



NWSUpdatePortal(myPortal);

NWSUpdatePortal(statusPortal);



NWSInitList (handle, free);



for (i=MSG_UPDATE;i<=MSG_ENCRYPT;i++)

  NWSAppendToList (messages[i],(void *) 0, handle);



NWSList (MSG_LIST, 5, 40, 4, 50,

                M_NO_SORT | M_SELECT , NULL, handle, NULL, ListAction, 0);



NWSDestroyList (handle);

NWSDestroyPortal (infoPortalNumber, handle);

}



/* cron thread, runs every minute */



void cronThread(void *p)

{

int i;

time_t t;

long tdiv;



do {

   delay(60000);  // wait 60 seconds

   ThreadSwitchWithDelay();

   t=time(NULL);

   for (i=CR_REMOVE;i<=CR_SMART;i++)

     {

     tdiv=t-cronTab[i].startTime;

     if (tdiv>=0 && (tdiv < cronTab[i].interval ||

         t-cronTab[i].lastTime> cronTab[i].interval))

       {

       cronTab[i].startTime+=cronTab[i].interval;

       cronTab[i].cronfunc();

       cronTab[i].lastTime=time(NULL);

       }

     }

   } while(1);

}



/* cron thread, runs every minute */



void oldcronThread(void *p)

{

time_t t;

struct tm tnow;



t=time(NULL);

localtime_r (&t,&tnow);



do {

   delay(60000);  // wait one minute

   ThreadSwitch();

   t=time(NULL);

   localtime_r (&t,&tnow);

   if (tnow.tm_min+60*tnow.tm_hour < crontab.activationTime.minutes+60*crontab.activationTime.hours)

     crontab.done=0;

   else

     if (!crontab.done)

       {

       portalPrint("Autocleanup");

       removeGroupMembers();

       crontab.done=1;

       }

   updateConfiguration();

   } while(1);

}



void clearThread(void *p)

{

for (;;) {

  if (clearRequest)

    {

    removeGroupMembers();

    clearRequest=0;

    }

  else

    {

    delay(1000);

    ThreadSwitchWithDelay();

    }

  }

}



char *timeStr(void)

{

static char tmp[16];

time_t t;

struct tm tnow;

t=time(NULL);

localtime_r (&t,&tnow);



strftime(tmp,16,"%H:%M:%S",&tnow);

return tmp;

}



char *dateStr(void)

{

static char tmp[16];

time_t t;

struct tm tnow;

t=time(NULL);

localtime_r (&t,&tnow);



strftime(tmp,16,"%d.%m.%y",&tnow);

return tmp;

}



/* update status display */



int updateStatus(void)

{

char tmp[128];

memberCount=0;

scanGroupMembers(removeInactiveUser);

sprintf(tmp,"%s %d nodes, %d active users, addCount: %ld, remCount: %ld",

timeStr(),nodeCount,memberCount,addCount,remCount);

statusPrint(tmp);

return 0;

}



void main(int argc, char *argv[])

{

int cC,i,nutThreadID,clearThreadID,cronThreadID;

char objectName[256],buf[80],pass[32],cName[256];

BYTE loginTime[10];

WORD connection,objectType;

long objectID;

NODE *ntmp;

INTERVAL iv;

time_t t,t1;

struct tm tnow;



dContext = NWDSCreateContext( );

if (dContext==ERR_CONTEXT_CREATION)

  {

  ConsolePrintf("Error creating context\n");

  return;

  }



NWDSSetContext(dContext,DCK_NAME_CONTEXT,argc==1 ? "O=uni-regensburg.c=de": argv[1] );

if (!NWDSGetContext(dContext, DCK_NAME_CONTEXT, cName))

  logPrintf("GROUPD context now: %s\n",cName);



nodes=(NODE*) calloc(MAX_NODE,sizeof(NODE));

newNodes=(NODE*) calloc(MAX_NODE,sizeof(NODE));



if (nodes==NULL || newNodes==NULL)

  {

  logPrintf("Error allocation node buffers\n");

  return;

  }



if (cC=readConfigFile(FALSE))

  {

  logPrintf("Error %x reading config file %s\n",cC,CONFIG);

  return;

  }



t=time(NULL);

localtime_r (&t,&tnow);

makeInterval(gParam.cleanupTime,&iv,1);



tnow.tm_sec=0; tnow.tm_min=iv.minutes; tnow.tm_hour=iv.hours;



t1=mktime(&tnow);

if (t1<t-60)

  t1+=86400;



for (i=CR_REMOVE;i<=CR_SMART;i++)

  cronTab[i].startTime=t;



cronTab[CR_REMOVE].startTime=t1;



if (initNUT())

  {

  logPrintf("Error initializing NUT\n");

  return;

  }



if (cC=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &readBuf))

  {

  logPrintf("NWDSAllocBuf: %x\n",cC);

  }



if (cC=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &aBuf))

  {

  logPrintf("NWDSAllocBuf: %x\n",cC);

  }



if (cC=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &aBuf2))

  {

  logPrintf("NWDSAllocBuf: %x\n",cC);

  }



if (cC=NWDSAllocBuf(DEFAULT_MESSAGE_LEN, &netaBuf))

  {

  logPrintf("NWDSAllocBuf: %x\n",cC);

  }



ntmp=nodes; nodes=newNodes; newNodes=ntmp;  // swap node arrays

nodeCount=newNodeCount;

qsort(nodes,nodeCount,sizeof(NODE),nodecmp);



sprintf(buf,"Login as %s",gParam.userName);

messages[15]=buf;



if (*gParam.encryptedPassWord==0)

  {

  NWSPromptForPassword (15,10,40,60,pass,FALSE,handle);

  strcpy(gParam.encryptedPassWord,encrypt(pass,gParam.userName));

  }

else

  strcpy(pass,decrypt(gParam.encryptedPassWord,gParam.userName));



trapInPtr=trapOutPtr=0;       // Zero the pending trap queue pointers

myThreadID=GetThreadID();     // Get this NLM's thread ID.

threadState=1;                // Set my status as running (see trap functions)



//   Activate the traps!



loginEventHandle=RegisterForEvent(EVENT_LOGIN_USER,LoginTrapHandler,NULL);

logoutEventHandle=RegisterForEvent(EVENT_LOGOUT_CONNECTION,LogoutTrapHandler,NULL);



atexit(cleanup);



cC=NWDSLogin(dContext,0,gParam.userName,pass,0);

if (cC)

  {

  logPrintf("NWDSLogin: %x\n",cC);

  return;

  }



if ((nutThreadID=BeginThread(nutThread,NULL,8192,NULL))==EFAILURE)

    {

    printf("Error creating NUT thread\n");

    }

  else

    RenameThread(nutThreadID,"GROUPD_NUT");

if ((cronThreadID=BeginThread(cronThread,NULL,8192,NULL))==EFAILURE)

  {

  printf("Error creating cron thread\n");

  }

  else

RenameThread(cronThreadID,"GROUPD_Cron");



if ((clearThreadID=BeginThread(clearThread,NULL,8192,NULL))==EFAILURE)

  {

  printf("Error creating cron thread\n");

  }

  else

RenameThread(clearThreadID,"GROUPD_Clear");



updateStatus();

sprintf(buf,"%s GROUPD started",timeStr());

portalPrint(buf);



for(;;)

  {

  while(trapInPtr==trapOutPtr)       // If no events in queue, go to sleep.

    {

    threadState=0;                // trap handlers wake the thread if

    SuspendThread(myThreadID);    // anything happens.

    }



  threadState=1;                          // If traps happen now, let

                                          // them know we're already awake.

  trapOutPtr=(trapOutPtr+1)%MAX_EVT;      // advance pointer

  connection=trapConnection[trapOutPtr];  // get connection of this event



  if(connection<1)

    continue;                          // Ignore log in or out

                                       // of NLM service connections.



  switch(trapType[trapOutPtr])            // Service the queued trap.

    {

    case EVENT_LOGIN_USER:

      GetConnectionInformation(connection,objectName,&objectType,&objectID,loginTime);

      if (badName(objectName))

        break;

      cC=AddUserToGroup(objectName,connection);

      break;

    }

  }

}

