#include <stdio.h>

#include <stdlib.h>

#include <string.h>

#include <conio.h>

#include <errno.h>

#include <ctype.h>

#include <nwqueue.h>

#include <nwbindry.h>

#include <nwmsg.h>

#include <niterror.h>

#include <nwdir.h>

#include <process.h>

#include <time.h>

#include <nwmisc.h>

#include "../inc/jobdef.h"

#include <nwenvrn.h>

#include <nwdsapi.h>

#include <nwcntask.h>

#include <nwconn.h>



#define MAXQ 32

#define VERSION "IQM NDS 1.00"

#define LVERSION "Interactive Queue Manager NDS 1.00"

#define PARAMFILE "DEFAULT"   /* default parameter file name */

#define MAXROW 24

#define MAXJOB 250

#define MAXAGE 60  /* default value for maxAge */

#define MAXJPS 8 /* maximum of jobs that can be displayed on one screen */

#define ONESEC 1000



#define BEEP putchar(7)

#define CLREOL printf("%-78s\r"," ")



typedef struct {

               char queueName[48];  /* its name */

               long queueID;        /* ID of queue */

               int autoDelete;      /* if true, jobs older than maxage will be

                                       automatically deleted */

               long maxAge;         /* maximum age before jobs will be deleted */

               long deleteCount;    /* jobs automatically deleted yet */

               int autoDeleteNonPS; /* if true, non PS jobs will be

                                       automatically deleted when auto delete

                                       thread runs */

               } QINFO;



int del=50,    /* delay for screen update in 100 ms units */

    myScreen,  /* screen handle */

    run=FALSE;



NWDSContextHandle   dContext;

WORD conn;



/* read password from console */



char *readpass(void)

{

static char pass[48]="\0";

int i=0;



while (i<48)

  {

  switch (pass[i]=getch()) {

    case 3: return(NULL);

    case 8: if (!i)

              putchar(7);

            else

              {

              printf("\b \b");

              i--;

              }

            break;

    case 13 : pass[i]=0; i=48; break;

    default : if (pass[i]>' ')

                {

                i++;

                putchar('_');

                }

              break;

    }

  }

printf("\n");

return(strupr(pass));

}



/* read in long integer without scanf */



long getlong(void)

{

char x[32];

*x=0;

fgets(x,32,stdin);

return(atol(x));

}



/* Functions for loading and writing parameters from and to file

   Format of parameter file:

   #REFRESH=<value>

   #QUEUE=<queuename>,<maxage>,<autodelete>

   any line which starts not with #QUEUE or #REFRESH is ignored */



#define KEY_QUEUE "QUEUE"

#define KEY_REFRESH "REFRESH"

#define DELIM "\n\r\t, "



/* load parameter from file */



int loadParameter(char *filename,QINFO *q)

{

char tmp[128],*p,*qn,*ma,*ad,*de,*dnps;

int i;

FILE *param;

sprintf(tmp,"SYS:SYSTEM\\%s\.IQM",filename);

if ((param=fopen(tmp,"r"))==NULL)

  return(-1);

while (!feof(param)) {

  if (fgets(tmp,128,param)==NULL)

    break;

  if (tmp[0]=='#')  /* parameter ? */

    if (p=strtok(tmp+1,"="))

      {

      if (!stricmp(p,KEY_REFRESH))

        {

        if (de=strtok(NULL,DELIM))

          {

          del=atoi(de)*10;

          del= del ? del : 50;

          }

        }

      else

        if (!stricmp(p,KEY_QUEUE))

          {

          if (qn=strtok(NULL,DELIM))

            if (ma=strtok(NULL,DELIM))

              if (ad=strtok(NULL,DELIM))

                {

                dnps=strtok(NULL,DELIM);

                for (i=0;i<MAXQ;i++)

                  if (q[i].queueID)

                    if (!stricmp(qn,q[i].queueName))

                      {

                      q[i].maxAge=atol(ma);

                      q[i].autoDelete=atoi(ad);

                      q[i].autoDeleteNonPS=dnps ? atoi(dnps) : 0;

                      break;

                      }

                 }

        }

      }

    }

fclose(param);

return(0);

}



/* save parameter to file */



int writeParameter(char *filename,QINFO *q)

{

char tmp[64];

FILE *param;

int i;

sprintf(tmp,"SYS:SYSTEM\\%s\.IQM",filename);

if ((param=fopen(tmp,"w"))==NULL)

  return(-1);

fprintf(param,"%s Parameter File\n",VERSION);

fprintf(param,"Syntax: #QUEUE=<queuename>,<maxage>,<autodel flag>,<autodel non PS flag>\n");

fprintf(param,"#%s=%d\n",KEY_REFRESH,del/10);

for (i=0;i<MAXQ;i++,q++)

  {

  if (q->queueID==NULL)

    break;

  fprintf(param,"#%s=%s,%ld,%i,%i\n",KEY_QUEUE,q->queueName,q->maxAge,

          q->autoDelete,q->autoDeleteNonPS);

  }

fclose(param);

return(0);

}



/* Calculate the difference between current time and job entry time in minutes.

   Only day, hour and minute are used, so the maximum age must be

   within 31 days */



long jobAge (BYTE *eT)

{

BYTE fsdt[7];

GetFileServerDateAndTime(fsdt);

return(abs((long)eT[2]*1440+(long)eT[3]*60+eT[4]-

            (long)fsdt[2]*1440-(long)fsdt[3]*60-fsdt[4]));

}



/* remove job if older than maxage */



void removeOldJob(QINFO *q,WORD jobNumber)

{

JobStruct job;



if (NWQGetJobEntry(q->queueID,jobNumber,&job))

  return;

if (job.jobControlFlags & QF_ENTRY_OPEN) /* leave open jobs in queue */

  return;





if (jobAge(job.jobEntryTime)>q->maxAge)

  if (!RemoveJobFromQueue(q->queueID,jobNumber))

    q->deleteCount++;

}



/* remove non PS job */



int removeNonPSJob(long queueID,WORD jobNumber)

{

JobStruct job;

char tmp[128];

WORD station[1];

BYTE result[1];



if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return 42;

if (job.jobControlFlags & QF_ENTRY_OPEN) /* leave open jobs in queue */

  return 42;

station[0]=job.clientStation;

if (!RemoveJobFromQueue(queueID,jobNumber))

  {

  sprintf(tmp,"Non PS job (%s) removed",job.textJobDescription);

  SendBroadcastMessage(tmp,station,result,1);

  return 0;

  }

}



/* The auto delete thread runs every 30 seconds */



void autoDeleteThread(QINFO *q)

{

int i;

WORD jobCount,jobList[MAXJOB];



while (run)

  {

  for (i=0;(i<30) && run;i++)

    delay(ONESEC);

  if (q->autoDelete || q->autoDeleteNonPS)

    if (!GetQueueJobList(q->queueID,&jobCount,jobList,MAXJOB))

      {

      for (i=0;i<jobCount;i++)

        if (q->autoDelete)

          removeOldJob(q,jobList[i]);

        else // remove non PS job

          if (!isPSjob(q->queueID,jobList[i]))

            {

            if (!removeNonPSJob(q->queueID,jobList[i]))

              q->deleteCount++;

            }

      }

  }

}



/* main help screen */



void mainhelp(void)

{

clrscr();

printf("%s Main help screen\n\n",VERSION);

printf("h:       Display this help screen\n");

printf("L:       Load configuration from file\n");

printf("q:       Quit\n");

printf("r:       Refresh screen\n");

printf("SN:      Set new filename for configuration\n");

printf("SR:      Set new screen refresh value\n");

printf("W:       Write configuration to file\n");

printf("+/-:     Select next/previous 8 queues\n");

printf("0 .. 7 : Show detailed info and manage queue jobs\n");

printf("\n");

PressAnyKeyToContinue();

}



/* prompt and return pressed key if key was in select or 0 else */



char userprompt(char *prompt,char *select)

{

char *p=select;

BYTE save[160]; /* buffer for one line */

CopyFromScreenMemory(1,80,save,0,MAXROW);

gotoxy(0,MAXROW); printf("%-70s\r%s ("," ",prompt);

while (*p)

  {

  printf("%c",*p++);

  if (*p)

    printf("/");

  }

printf(")? ");

p=strchr(select,toupper(getch()));

CopyToScreenMemory(1,80,save,0,MAXROW);

if (p)

  return(*p);

else

  return(0);

}



/* display manage help screen */



void managehelp(QINFO *q)

{

clrscr();

printf("%s Manage help screen\n\n",VERSION);

printf("DQ:      Disable current queue\n");

printf("DS:      Disable print server for current queue\n");

printf("EQ:      Enable current queue\n");

printf("ES:      Enable print server for current queue\n");

printf("F:       Free a held job\n");

printf("h:       Display this help screen\n");

printf("H:       Hold jobs\n");

printf("K:       Kill jobs\n");

printf("q:       Return to main screen\n");

printf("r:       Refresh screen\n");

printf("SA:      Set maximum job age before deleting (current: %ld min)\n",q->maxAge);

printf("SD:      Set auto deletion of old jobs (current: %s)\n",q->autoDelete ? "ON" : "OFF");

printf("SP:      Set auto deletion of non PS jobs (current: %s)\n",q->autoDeleteNonPS ? "ON" : "OFF");

printf("v:       View first 256 bytes of print job\n");

printf("+/- :    Display next or previos %d jobs\n",MAXJPS);

printf("0 .. 7 : Switch to other queue\n");

printf("\n");

PressAnyKeyToContinue();

}



/* convert queue status byte to string */



char *qstatstr(BYTE stat)

{

static char s[128];

*s=0;

if (!stat)

  strcpy(s,"[Ready]");

else

  {

  strcpy(s,"[");

  if (stat & QS_CANT_ADD_JOBS)

    strcat(s,"Adding disabled ");

  if (stat & QS_SERVERS_CANT_ATTACH)

    strcat(s,"No server attach ");

  if (stat & QS_CANT_SERVICE_JOBS)

    strcat(s,"No service");

  strcat(s,"]");

  }

return(s);

}



/* print queue status */



int qstatus(QINFO *q)

{

BYTE jobCnt,qStat,servCnt;



printf("Queue %-16s, Autodel %s (>%ld min), %s jobs deleted: %ld\n",

        q->queueName,q->autoDelete ? "ON" : "OFF",q->maxAge,

        q->autoDeleteNonPS ? "del non PS," : "",q->deleteCount);

if (!ReadQueueCurrentStatus(q->queueID,&qStat,&jobCnt,&servCnt,NULL,NULL,0))

      printf("   Status %s, %3d job(s)\n",

             qstatstr(qStat),jobCnt);

return qStat;

}



int     printJobInfo(long queueID,WORD jobNumber)

{

char    userName[48];

WORD    oType;

long    fileSize;

BYTE    *b;

int     jobOpen,cC;



NWQJobRec_t *jobp;

JobStruct job;





jobOpen=0;

jobp=&job;

CLREOL; printf("Job %d",jobNumber);

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  printf(":\n");

else

  {

  b=(BYTE *) job.jobEntryTime;

  printf(" [%3d]:",job.jobPosition);

  if (! GetBinderyObjectName(LongSwap(jobp->client.id),userName,&oType))

    {

    printf(" User %s, ",userName);

    printf("%d.%d.%02d %d:%02d.%02d (age: %ld min) ",b[2],b[1],b[0],b[3],b[4],b[5],

    jobAge(job.jobEntryTime));

    }

  else

    {

    sprintf(userName,"%08lx",job.clientIDNumber);

    cC=NWDSMapIDToName(dContext,conn,/*LongSwap*/(job.clientIDNumber),userName);

    printf(" User %s , ",userName,cC);

    printf("%d.%d.%02d %d:%02d.%02d (age: %ld min) ",b[2],b[1],b[0],b[3],b[4],b[5],

    jobAge(job.jobEntryTime));

    }

  if (job.serverIDNumber != NULL)

    {

    printf("[active] ");

    }

  else

    if (! (job.jobControlFlags & ~0x1f) )

      printf("[ready] ");

  if (job.jobControlFlags & QF_ENTRY_OPEN)

    {

    printf("[adding] ");

    jobOpen = 1;

    }

  if (job.jobControlFlags & QF_USER_HOLD)

    printf("[user hold] ");

  if (job.jobControlFlags & QF_OPERATOR_HOLD)

    printf("[op. hold] ");

  if (isPSjob(queueID,jobNumber))

    printf("PS");

  printf("\n"); CLREOL;

  if (! GetQueueJobsFileSize(queueID,jobNumber,&fileSize))

    printf("   (%s) %ld byte(s) %d copies\n",job.textJobDescription,fileSize,

    IntSwap(((CRA*)job.clientRecordArea)->nrOfCopies));

  }

return job.jobControlFlags; //jobOpen;

}



/* kill non open job from queue */



void killjob(long queueID,WORD jobNumber)

{

JobStruct job;

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return;

if (job.jobControlFlags & QF_ENTRY_OPEN)

  return;

RemoveJobFromQueue(queueID,jobNumber);

}



/* set job to operator hold */



void holdjob(long queueID,WORD jobNumber)

{

JobStruct job;

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return;

job.jobControlFlags |= QF_OPERATOR_HOLD;

ChangeQueueJobEntry(queueID,&job);

}



/* remove operator hold from job */



void freejob(long queueID,WORD jobNumber)

{

JobStruct job;

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return;

job.jobControlFlags &= ~QF_OPERATOR_HOLD;

ChangeQueueJobEntry(queueID,&job);

}



/* view first 256 byte of non active job */



void viewjob(long queueID,WORD jobNumber)

{

JobStruct job;

char tmp[256],qdir[128],qname[48];

int qType,i,j,n;

FILE *f;



if (GetBinderyObjectName(queueID,(BYTE *)qname,&qType))

  return;

if (ReadPropertyValue(qname,qType,"Q_DIRECTORY",1,qdir,NULL,NULL))

  return;

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return;

sprintf(tmp,"%s\\%s",qdir,job.jobFileName);

if (f=fopen(tmp,"rb"))

  {

  clrscr();

  n=fread(tmp,1,256,f);

  for (i=0;i<n;i+=16)

    {

    printf("\n%04X: ",i);

    for (j=0;j<16;j++)

      printf("%02X ",(BYTE) tmp[i+j]);

    for (j=0;j<16;j++)

      printf("%c",tmp[i+j]>31 ? tmp[i+j] : '.');

    }

  fclose(f);

  printf("\n\n");

  PressAnyKeyToContinue();

  }

}



/* return true if job file contains postscript */



int isPSjob(long queueID,WORD jobNumber)

{

JobStruct job;

char tmp[256],qdir[128],qname[48],*p;

int qType,isPS=1;

FILE *f;



if (GetBinderyObjectName(queueID,(BYTE *)qname,&qType))

  return isPS;

if (ReadPropertyValue(qname,qType,"Q_DIRECTORY",1,qdir,NULL,NULL))

  return isPS;

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  return isPS;

sprintf(tmp,"%s\\%s",qdir,job.jobFileName);

if (f=fopen(tmp,"rb"))

  {

  *tmp=0;

  fread(tmp,1,256,f);

  fclose(f);

  isPS=0;

  if (p=strtok(tmp,"\004\n\r"))

    if (!strncmp(p,"%!",2))

      isPS=1;

  }

return isPS;

}



/* Display full information and manage queue jobs */



void manageQueue(QINFO *q)

{

int i,firstJob=0,j,prompt;

char c;

WORD jobCount,jobList[MAXJOB];

time_t t;

long l;

int qStat;



do {

  CheckIfScreenDisplayed(myScreen,TRUE);  /* wait until screen is displayed */

  clrscr();

  time(&t);

  printf("Current time : %s",ctime(&t));

  qStat=qstatus(q);

  if (!GetQueueJobList(q->queueID,&jobCount,jobList,MAXJOB))

    {

    if (jobCount<=firstJob)

      firstJob=max(0,firstJob-MAXJPS);

    for (i=firstJob;i<firstJob+MAXJPS;i++)

      if (i<jobCount)

        printJobInfo(q->queueID,jobList[i]);

    }

  gotoxy(0,MAXROW);

  printf("q=return to main, h=Help, r=refresh screen (default is every %d s)",del/10);

  for (i=0;i<del;i++)

    {

    delay(100);

    if (kbhit())

      switch (/*toupper*/(c=getch())) {

        case 'D' : c=userprompt("Disable Queue, Server","QS");

          {

          switch (c) {

          case 'Q' : if (userprompt("Disable Queue","YN")=='Y')

                     {

                     SetQueueCurrentStatus(q->queueID,QS_CANT_ADD_JOBS);

                     i=del;

                     }

                   break;

          case 'S' : if (userprompt("Disable Server","YN")=='Y')

                     {

                     SetQueueCurrentStatus(q->queueID,QS_CANT_SERVICE_JOBS);

                     i=del;

                     }

                   break;

                   }

           break;

           }

        case 'E' : c=userprompt("Enable Queue, Server","QS");

          {

          switch (c) {

          case 'Q' : if (userprompt("Enable Queue","YN")=='Y')

                     {

                     SetQueueCurrentStatus(q->queueID,qStat & ~QS_CANT_ADD_JOBS);

                     i=del;

                     }

                   break;

          case 'S' : if (userprompt("Enable Server","YN")=='Y')

                     {

                     SetQueueCurrentStatus(q->queueID,qStat & ~QS_CANT_SERVICE_JOBS);

                     i=del;

                     }

                   break;

                   }

           break;

           }

        case 'K' : for (j=firstJob,prompt=1;j<jobCount;j++)

                     {

                     if (prompt)

                       {

                       gotoxy(0,20);

                       printJobInfo(q->queueID,jobList[j]);

                       c=userprompt("Kill job(s)","YNAQ");

                       if (c=='Q')

                         break;

                       if (c=='A')

                         {

                         killjob(q->queueID,jobList[j]);

                         prompt=0;

                         }

                       if (c=='Y')

                         killjob(q->queueID,jobList[j]);

                       }

                     else

                       killjob(q->queueID,jobList[j]);

                     }

                   i=del; break;

        case 'F' : for (j=firstJob,prompt=1;j<jobCount;j++)

                     {

                     if (prompt)

                       {

                       gotoxy(0,20);

                       printJobInfo(q->queueID,jobList[j]);

                       c=userprompt("Free job(s)","YNAQ");

                       if (c=='Q')

                         break;

                       if (c=='A')

                         {

                         freejob(q->queueID,jobList[j]);

                         prompt=0;

                         }

                       if (c=='Y')

                         freejob(q->queueID,jobList[j]);

                       }

                     else

                       freejob(q->queueID,jobList[j]);

                     }

                   i=del; break;

        case 'H' : for (j=firstJob,prompt=1;j<jobCount;j++)

                     {

                     if (prompt)

                       {

                       gotoxy(0,20);

                       printJobInfo(q->queueID,jobList[j]);

                       c=userprompt("Hold job(s)","YNAQ");

                       if (c=='Q')

                         break;

                       if (c=='A')

                         {

                         holdjob(q->queueID,jobList[j]);

                         prompt=0;

                         }

                       if (c=='Y')

                         holdjob(q->queueID,jobList[j]);

                       }

                     else

                       holdjob(q->queueID,jobList[j]);

                     }

                   i=del; break;

        case 'v' : for (j=firstJob,prompt=1;j<jobCount;j++)

                     {

                     if (prompt)

                       {

                       gotoxy(0,20);

                       printJobInfo(q->queueID,jobList[j]);

                       c=userprompt("View job","YNQ");

                       if (c=='Q')

                         break;

                       if (c=='Y')

                         viewjob(q->queueID,jobList[j]);

                       }

                     else

                       viewjob(q->queueID,jobList[j]);

                     }

                   i=del; break;

        case 'q' : return;

        case 'r' : i=del; break;

        case 'S' : switch (c=userprompt("Set what","ADP")) {

                     case 'D':

                       {

                       switch (userprompt("Set auto deletion","01Q")) {

                         case '0' : q->autoDelete=FALSE; break;

                         case '1' : q->autoDelete=TRUE; break;

                         }

                       break; // 'D'

                       }

                     case 'P':

                       {

                       switch (userprompt("Set auto deletion of non PS jobs","01Q")) {

                         case '0' : q->autoDeleteNonPS=FALSE; break;

                         case '1' : q->autoDeleteNonPS=TRUE; break;

                         }

                       break; // 'P'

                       }

                     case 'A':

                       {

                       gotoxy(0,MAXROW); CLREOL;

                       printf("Enter maximum age: ");

                       if (l=getlong())

                         q->maxAge=l;

                       break; // 'A'

                       }

                   } // 'S'

                   i=del; break;

        case 'h' : managehelp(q); i=del; break;

        case '+' : if ((firstJob+MAXJPS)<jobCount)

                     {

                     firstJob+=MAXJPS; i=del;

                     }

                   break;

        case '-' : if (firstJob)

                     {

                     firstJob-=MAXJPS; i=del;

                     }

                   break;

        default: if (isdigit(c)) { ungetch(c); return; }

           BEEP;

        }

    }

  } while (1);

}



void main(int argc,char *argv[])

{

int     sCC,queueCnt=0,i,n,firstQueue=0;

long    l,ID=-1l;

char    c,screenName[80],config[16],dummy[16],serverName[50];

WORD    objectType,fileServerID;

QINFO   *q,qinfo[MAXQ];



dContext = NWDSCreateContext( );



GetServerInformation(48,(FILE_SERV_INFO *)serverName);

GetFileServerID("RZI", &fileServerID);



//conn = GetCurrentConnection();

conn=fileServerID;

// NWDSAuthenticate(conn,0, NULL);

// NWDSLogin(dContext, 0, "ADMIN","ZUKURZ",0);



sprintf(screenName,"%s %s %d Screen",VERSION,serverName,conn);

strcpy(config,PARAMFILE);

if ((myScreen=CreateScreen(screenName,AUTO_DESTROY_SCREEN))==EFAILURE)

  return;

DisplayScreen(myScreen);

for (i=0;i<MAXQ;i++)

  qinfo[i].queueID=NULL;

while (!(sCC = ScanBinderyObject("*",OT_PRINT_QUEUE,&ID,

                         qinfo[queueCnt].queueName,&objectType,dummy,dummy,dummy)))

  {

  q=&qinfo[queueCnt];

  q->autoDelete=FALSE;

  q->autoDeleteNonPS=FALSE;

  q->maxAge=MAXAGE;

  q->queueID=ID;

  q->deleteCount=0;

  queueCnt++;

  if (queueCnt==MAXQ)

    break;

  }



if (!queueCnt)

  {

  printf("No queues found\n");

  PressAnyKeyToContinue();

  return;

  }

qsort(qinfo,queueCnt,sizeof(QINFO),stricmp);



/* if parameter file was given, set values */



if (argc>1)

  {

  if (strlen(argv[1])<9)

    {

    strcpy(config,argv[1]);

    loadParameter(strupr(config),qinfo);

    }

  }



del = del ? del : 50;

for (i=0;i<queueCnt;i++)

if (BeginThread(autoDeleteThread,NULL,4096,&qinfo[i])==EFAILURE)

  {

  printf("Error creating thread for queue %s\n",qinfo[i].queueName);

  PressAnyKeyToContinue();

  return;

  }



run=TRUE;



do {

  CheckIfScreenDisplayed(myScreen,TRUE);  /* wait until screen is displayed */

  clrscr();

  printf("%s Main Screen, param file: %s\n\n",VERSION,config);

  for (i=firstQueue,q=qinfo+firstQueue;i<min(queueCnt,firstQueue+8);i++,q++)

    {

    /* gotoxy(0,i*2); */

    printf("%d%d:",i/8,i%8); qstatus(q);

    }

  gotoxy(0,MAXROW);

  printf("q=end, h=Help, r=refresh screen (default is every %d s)",del/10);

  for (i=0;i<del;i++)

    {

    delay(100);

    if (kbhit())

      switch (/*toupper*/(c=getch())) {

        case 'h' : mainhelp(); i=del; break;

        case 'L' : if (userprompt("Load parameters from file","YN")=='Y')

                     if (loadParameter(config,qinfo))

                       BEEP;

                   i=del; break;

        case 'q' : {

          if (userprompt("Quit","YN")=='Y')

            {

            run=FALSE; i=del;

            }

          break;

          }

        case 'r' : i=del; break;

        case 'S' : switch(userprompt("Set what","NR")) {

                     case 'N' : {

                                gotoxy(0,MAXROW); CLREOL;

                                printf("Enter new file name: ");

                                if (!fgets(config,10,stdin))

                                  strcpy(config,PARAMFILE);

                                else

                                  strtok(strupr(config),"\n\r");

                                i=del;

                                } break;

                     case 'R' : {

                                gotoxy(0,MAXROW); CLREOL;

                                printf("Enter new refresh value: ");

                                if (l=getlong())

                                  del=(int)l*10;

                                i=del;

                                } break;

                             } break;

        case 'W' : if (userprompt("Write parameters to file","YN")=='Y')

                     if (writeParameter(config,qinfo))

                       BEEP;

                   i=del; break;

        case '+' : if (firstQueue+8<queueCnt) { firstQueue+=8; i=del;}

                     break;

        case '-' : if (firstQueue) { firstQueue-=8; i=del;}

                     break;

        default: if (isdigit(c) && (c<'8'))

          {

          n=firstQueue+(c & 0x7);

          if (n<queueCnt)

            {

            manageQueue(&qinfo[n]);

            i=del;

            }

          else

            BEEP;

          }

        else

          BEEP;

        }

    }

  } while (run);

NWDSLogout(dContext);

ReturnConnection(conn);

NWDSFreeContext(dContext);

}

