/*------------------------------------------------------------------------

**      Module: NDEMO v1.00

**

**      Source: Novell Engineering

**

**      Abstract:

**              This module shows how to create an NLM that uses

**              the NWSNUT Interface.  This is the interface used

**              by many Novell NLMs, such as Monitor and Install.

**              Examples of several NWSNUT features are given,

**              including menus, forms, context specific help,

**              and edit boxes.

**

**      Environment:

**              NetWare 3.x and 4.x

**              Network C for NLMs SDK

**              NetWare CLib Runtime 3.x and 4.x

**

**      This software is provided as is and carries no warranty

**      whatsoever.  Novell disclaims and excludes any and all implied

**      warranties of merchantability, title and fitness for a particular

**      purpose.  Novell does not warrant that the software will satisfy

**      your requirements or that the software is without defect or error

**      or that operation of the software will be uninterrupted.  You are

**      using the software at your risk.  The software is not a product

**      of Novell, Inc. or any of subsidiaries.

**

**      Functions used by NDEMO

**      --------------------------------------------------------------

**        AllocateResourceTag

**        AtUnload

**        CreateScreen

**        DestroyScreen

**        DisplayScreen

**        free

**        GetNLMHandle

**        GetSizeOfScreen

**        NWSAlert

**        NWSAppendBoolField

**        NWSAppendCommentField

**        NWSAppendHexField

**        NWSAppendHotSpotField

**        NWSAppendIntegerField

**        NWSAppendMenuField

**        NWSAppendStringField

**        NWSAppendToList

**        NWSAppendToMenu

**        NWSAppendToMenuField

**        NWSAppendUnsignedIntegerField

**        NWSClearPortal

**        NWSConfirm

**        NWSCreatePortal

**        NWSDestroyForm

**        NWSDestroyList

**        NWSDestroyMenu

**        NWSDestroyPortal

**        NWSEditPortalForm

**        NWSEditString

**        NWSEditText

**        NWSGetKey

**        NWSGetMessage

**        NWSGetPCB

**        NWSInitForm

**        NWSInitializeNut

**        NWSInitList

**        NWSInitMenu

**        NWSInitMenuField

**        NWSList

**        NWSMenu

**        NWSPopHelpContext

**        NWsprintf

**        NWSPushHelpContext

**        NWSRestoreNut

**        NWSSelectPortal

**        NWSSetDynamicMessage

**        NWSShowPortalLine

**        NWSUpdatePortal

**        printf

**        strlen

**      --------------------------------------------------------------

**

**------------------------------------------------------------------------

*/

#include <stdio.h>

#include <conio.h>

#include <advanced.h>

#include <process.h>

#include <time.h>

#include <nwsnut.h>

#include <string.h>

#include <malloc.h>



#include "ndemo\ndemo.hlh"                    /*****  help definitions  ******/

#include "ndemo\ndemo.mlh"                    /*****  message definitions  *******/





/******  Global variables in this module *******/



NUTInfo *handle;

LONG    NLMHandle;

LONG    allocTag;

int     CLIBScreenID;





/******  prototypes for functions in this module  *******/



void WriteInstructions (BYTE *p, PCB *pcb);

int ListAction (LONG keyPressed, LIST **elementSelected, LONG *itemNumber,

                        void *listParameter);

int  MenuAction(int option, void *parameter);

LONG HotSpotAction (FIELD *fp, int selectKey, int *changedField,

                        NUTInfo *handle);

int  FormMenuAction(int option, void *parameter);

void myunload(void);



extern void NWsprintf (void *, void *, ...);



/******  the main routine  *******/



void main()

{

        LONG    ccode, type, portalNumber, infoPortalNumber;

        int menuChoice, myInteger = 600, myHexInteger = 0x2ffc, line;

        LONG myOtherInteger = 900;

        PCB *pPtr, *ipPtr;

        BYTE value, *ptr, myBoolean, string[200];

        LONG    portalTop = 6,                          /* position at 6, 20 on physical screen */

                portalLeft = 20,

                portalFrameHeight = 10,          /*  make the portal 10 X 40 */

                portalFrameWidth = 40,

                portalVirtualHeight = 6,                /* make the virtual portal 6 X 38 */

                portalVirtualWidth = 38;

        WORD screenHeight, screenWidth;

        MFCONTROL *mfctl;





        NWsprintf (string, "howdy");





        /****** get a handle for allocating a resource tag *******/



        NLMHandle = GetNLMHandle();





        /****** create a screen for displaying our information *******/



        CLIBScreenID = CreateScreen("NWSNUT Demo Screen", AUTO_DESTROY_SCREEN);

        if (!CLIBScreenID)

                return;





        /******* allocate a resource tag to use for memory allocations *******/



        allocTag = AllocateResourceTag(NLMHandle, "NWSNUT DEMO Alloc Tag",

                                            AllocSignature);

        if (!allocTag)

        {

                DestroyScreen(CLIBScreenID);

                return;

        }





        /********  initialize NWSNUT ********/



        ccode = NWSInitializeNut(UTILITY_MSG, VERSION_320, NORMAL_HEADER,

                                        NUT_REVISION_LEVEL, 0, 0, CLIBScreenID, allocTag, &handle);

        if (ccode)

        {

                DestroyScreen(CLIBScreenID);

                return;

        }



        /******** setup a procedure to be called if we are unloaded from the

                       console with an UNLOAD XXXXXX command *********/



        AtUnload(myunload);





        /*******  activate the screen that we created *******/



        DisplayScreen(CLIBScreenID);





        /* create an information portal at bottom of screen,

                no border, no header, size = 1 X screenWidth */



        GetSizeOfScreen (&screenHeight, &screenWidth);



        infoPortalNumber = NWSCreatePortal((LONG) (screenHeight - 1), 0L,

                        1L, (LONG)screenWidth, 1L, (LONG) screenWidth, SAVE,

                        (BYTE *) 0, 0L, NOBORDER, 0, CURSOR_OFF, VIRTUAL, handle);



        if (infoPortalNumber > MAXPORTALS)

        {

                printf ("Unable to create information portal\n");

                DestroyScreen(CLIBScreenID);

                return;

        }



        /******** convert the portal number to a PCB pointer *********/



        NWSGetPCB (&ipPtr, infoPortalNumber, handle);





        WriteInstructions ("Press any key to create a portal", ipPtr);

        NWSGetKey (&type, &value, handle);







        /******* create a virtual portal *******/



        portalNumber = NWSCreatePortal(portalTop, portalLeft,

                portalFrameHeight, portalFrameWidth, portalVirtualHeight,

                portalVirtualWidth, SAVE, "Demonstration Portal", 0,

                DOUBLE, 0, CURSOR_ON, VIRTUAL, handle);



        if (portalNumber > MAXPORTALS)

        {

                NWSDestroyPortal (infoPortalNumber, handle);

                printf ("Unable to create virtual portal\n");

                DestroyScreen(CLIBScreenID);

                NWSRestoreNut(handle);

                return;

        }



        /******** convert this number to a PCB pointer as well ******/



        NWSGetPCB (&pPtr, portalNumber, handle);

        goto done;





        /******** clear the portal, and bring it to the front ******/



        NWSClearPortal (pPtr);

        NWSSelectPortal (portalNumber, handle);



        ptr = "This is a portal";

        NWSShowPortalLine (0, 0, ptr, strlen (ptr),      pPtr);



        ptr = "It may contain any type of data";

        NWSShowPortalLine (2, 0, ptr, strlen (ptr),      pPtr);



        NWSUpdatePortal (pPtr);                  /**** cause it to be displayed on the screen ****/



        WriteInstructions ("Press any key to create a List", ipPtr);

        NWSGetKey (&type, &value, handle);





        /******* cleanup and discard this portal *********/



        NWSDestroyPortal (portalNumber, handle);





        /******** create a list ********/



        WriteInstructions ("Press ESCAPE to exit, or select a list item with ENTER", ipPtr);



        NWSInitList (handle, free);



        NWSAppendToList (NWSGetMessage (LIST_ITEM_1, &(handle->messages)),

                (void *) 0, handle);

        NWSAppendToList (NWSGetMessage (LIST_ITEM_2, &(handle->messages)),

                (void *) 0, handle);

        NWSAppendToList (NWSGetMessage (LIST_ITEM_3, &(handle->messages)),

                (void *) 0, handle);

        NWSAppendToList (NWSGetMessage (LIST_ITEM_4, &(handle->messages)),

                (void *) 0, handle);





        NWSList (LIST_HEADER, 10, 40, 4,

                strlen (NWSGetMessage (LIST_HEADER, &(handle->messages))) + 4,

                M_ESCAPE | M_SELECT, NULL,      handle, NULL, ListAction, 0);





        /******** cleanup and discard this list ********/



        NWSDestroyList (handle);





        /******* create a menu *******/



        WriteInstructions ("Press ESCAPE to exit, or select a menu item with ENTER", ipPtr);



        NWSInitMenu (handle);



        /******** build the menu items on the fly. These could come from the

                                 message file just as easily. *********/



        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                                "Menu Item 1       ", &handle->messages);

        NWSSetDynamicMessage(DYNAMIC_MESSAGE_TWO,

                                                "Menu Item 2       ", &handle->messages);

        NWSSetDynamicMessage(DYNAMIC_MESSAGE_THREE,

                                                "Menu Item 3       ", &handle->messages);





        NWSAppendToMenu (DYNAMIC_MESSAGE_ONE, 1, handle);

        NWSAppendToMenu (DYNAMIC_MESSAGE_TWO, 2, handle);

        NWSAppendToMenu (DYNAMIC_MESSAGE_THREE, 3, handle);



        NWSMenu (MENU_HEADER, 10, 40, NULL, MenuAction, handle, (void *)handle);

        NWSDestroyMenu (handle);







        /******* create a form with various types of fields *******/



        WriteInstructions ("Press ESCAPE to exit, or select any field with ENTER", ipPtr);



        NWSInitForm (handle);



        line = 0;

        NWSAppendCommentField (line, 1, "Boolean Field:", handle);

        NWSAppendBoolField (line, 25, NORMAL_FIELD, &myBoolean, NULL, handle);



        line += 2;

        NWSAppendCommentField (line, 1, "Integer Field:", handle);

        NWSAppendIntegerField (line, 25, NORMAL_FIELD, &myInteger, 0, 9999,

                NULL, handle);



        line += 2;

        NWSAppendCommentField (line, 1, "String Field:", handle);

        strcpy (string, "Data String");

        NWSAppendStringField (line, 25, 30, NORMAL_FIELD, string, "A..Za..z ", NULL,

                handle);





        line += 2;

        NWSAppendCommentField (line, 1, "Unsigned Integer Field:", handle);

        NWSAppendUnsignedIntegerField (line, 25, NORMAL_FIELD, &myOtherInteger, 0, 99999,

                NULL, handle);



        line += 2;

        NWSAppendCommentField (line, 1, "Hex Field:", handle);

        NWSAppendHexField (line, 25, NORMAL_FIELD, &myHexInteger, 0, 99999,

                NULL, handle);



        line += 2;

        NWSAppendCommentField (line, 1, "Comment Field:", handle);

        NWSAppendCommentField (line, 25, "A comment", handle);



        line += 2;

        NWSAppendCommentField (line, 1, "Hot Spot Field:", handle);

        NWSAppendHotSpotField (line, 25, NORMAL_FIELD,

                "Hot Field", HotSpotAction, handle);





        mfctl = NWSInitMenuField (FORM_MENU_HEADER, 10, 40, FormMenuAction,

                handle);



        NWSAppendToMenuField (mfctl, MENU_TEXT_ONE, 1, handle);

        NWSAppendToMenuField (mfctl, MENU_TEXT_TWO, 2, handle);



        menuChoice = 1;                  /* display the text for option one */



        line += 2;

        NWSAppendCommentField (line, 1, "Menu Field:", handle);

        NWSAppendMenuField (line, 25, NORMAL_FIELD, &menuChoice, mfctl,

                NULL, handle);



        /* if no help is desired for the form, pass F_NO_HELP as the

                help parameter */



        NWSEditPortalForm (FORM_HEADER, 11, 40, 16, 50, F_NOVERIFY, FORM_HELP,

                EXIT_FORM_MSG, handle);



        /******* cleanup and discard this form *******/



        NWSDestroyForm (handle);







        /******* create a confirm box *******/



        WriteInstructions ("Press F1 to view a help screen", ipPtr);



        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                "Choose YES or NO and press ENTER", &handle->messages);





        /******* add some context specific help *******/



        NWSPushHelpContext (CONFIRM_BOX, handle);



        ccode = NWSConfirm(DYNAMIC_MESSAGE_ONE, 10, 40, 1,  NULL, handle, NULL);



        /******* remove the help *******/



        NWSPopHelpContext (handle);





        /****** create an EditString portal ************/





        WriteInstructions ("Use editing keys to edit the string, ENTER to exit", ipPtr);



        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                "String Edit Function", &handle->messages);



        NWSSetDynamicMessage(DYNAMIC_MESSAGE_TWO,

                " Your Name: ", &handle->messages);



        strcpy (string, "JOSEPH P. USER");



        ccode = NWSEditString(10, 40, 1, 40, DYNAMIC_MESSAGE_ONE,

                DYNAMIC_MESSAGE_TWO, string, sizeof (string), EF_ANY | EF_UPPER,

                handle, NULL, NULL, NULL);





        /******** create an EditText portal to allow user input *******/





        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                "Text Edit Function", &handle->messages);



        NWSSetDynamicMessage(DYNAMIC_MESSAGE_TWO,

                "Save changes to this text?", &handle->messages);



        strcpy (string, "This could be any kind of text\nthat you might have.");



        ccode = NWSEditText (10, 40, 4, 40, DYNAMIC_MESSAGE_ONE,

                string, sizeof (string), DYNAMIC_MESSAGE_TWO, 0, handle);







        /******* cleanup and discard the remaining portal *******/

done:

        NWSDestroyPortal (infoPortalNumber, handle);





        /******* allow NUT to cleanup any additional leftovers *******/



        NWSRestoreNut(handle);





        /******* and discard the new screen we created *******/



        DestroyScreen(CLIBScreenID);

}





void myunload(void)

{

        NWSRestoreNut(handle);

        DestroyScreen(CLIBScreenID);

}









void WriteInstructions (BYTE *p, PCB *pcb)

{

        NWSClearPortal (pcb);

        NWSShowPortalLine (0, 0, p, strlen (p), pcb);

        NWSUpdatePortal (pcb);

}





int ListAction (LONG keyPressed, LIST **elementSelected, LONG *itemNumber, void *listParameter)

{

        elementSelected = elementSelected;

        listParameter = listParameter;



        if (keyPressed == M_ESCAPE)

                return 1;



        switch ((*itemNumber) + 1)

        {

                case 1:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 1", &handle->messages);

                        break;



                case 2:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 2", &handle->messages);

                        break;



                case 3:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 3", &handle->messages);

                        break;



                case 4:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 4", &handle->messages);

                        break;



                default:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected another item", &handle->messages);

                        break;



        }



        NWSAlert (0, 0, handle, DYNAMIC_MESSAGE_ONE);





        return -1;

}





int  MenuAction(int option, void *junk)

{



        option = option;                        /* keep the compiler quiet */

        junk = junk;



        switch (option)

        {

                case 1:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 1", &handle->messages);

                        break;



                case 2:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 2", &handle->messages);

                        break;



                case 3:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 3", &handle->messages);

                        break;



                case 4:

                        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "You selected item number 4", &handle->messages);

                        break;



                default:

                        return 0;

        }



        NWSAlert (0, 0, handle, DYNAMIC_MESSAGE_ONE);



        return -1;



}



int  FormMenuAction(int option, void *parameter)

{

        parameter = parameter;          /* for the compiler */





        /* do anything that might be needed by the selection of a

                given menu option */







        /* and the value returned will indicate which data item is to be

                displayed in the menu field on the form */



        return option;

}



LONG HotSpotAction (FIELD *fp, int selectKey, int *changedField,

                        NUTInfo *handle)

{



        selectKey = selectKey;

        fp = fp;

        changedField = changedField;





        NWSSetDynamicMessage(DYNAMIC_MESSAGE_ONE,

                                "This is your hot spot routine", &handle->messages);



        NWSAlert (0, 0, handle, DYNAMIC_MESSAGE_ONE);



        return K_RIGHT;          /***** send us to the next field ******/

}





