/**************************************************************************/

/*                                                                        */

/*        Programm zum Testen der Performance eines Laufwerkes            */

/*        (c) Stephan Kammerer                                            */

/*                                                                        */

/**************************************************************************/



#include <stdio.h>

#include <time.h>

#include <stdlib.h>

#ifdef __FLAT__

#define NLM

#include <nwmisc.h> /* nur fr NLMs */

#else

#define LONG unsigned long

#define BYTE unsigned char

#endif



#include <io.h>

#include <fcntl.h>



clock_t anfang,ende;    /* Start und Endzeit in Timerticks */

FILE    *logfile,*file,*output;

BYTE    buffer[10240];



#define START anfang=clock() /* Startzeit nehmen */

#define STOP ende=clock()    /* Endzeit nehmen */

#define ONE_KB 1024



/* Differenz zischen Start- und Endzeit anzeigen */



void zeigeZeit(void)

{

clock_t t=ende-anfang;

#ifndef NLM

t=(t*1000l)/182l;     /* Umrechnung timer ticks in 100tel sec */

#endif

printf("Zeit: %ld.%02ld sec\n",t/100,t%100);

fprintf(logfile,"Zeit: %ld.%02ld sec\n",t/100,t%100);

}



/* Meldung auf Bildschirm und in Logdatei schreiben */



void log(char *msg)

{

printf("%s",msg);

fprintf(logfile,"%s",msg);

}

/*------------------------------------------------------------------------*/

/* Prozedur zum Erstellen einer Datei mit "laenge" Bytes und den Namen    */

/* "name".                                                                */

/*------------------------------------------------------------------------*/



void erzeuge_datei(long blocks, char *name)

{

  long i;



  if((file = fopen(name, "wb")) == NULL)

  {

    printf("ERROR: Kann datei nicht oeffnen\n");

    exit(1);

  }

  for (i=0;i<blocks;i++)

    if (fwrite (buffer, ONE_KB, 1, file) !=1)

      break;

  fclose(file);

}



/*------------------------------------------------------------------------*/

/* Prozedur zum Kopieren von "anzahl" Dateien. Diese sind haben           */

/* "blockanzahl" Blocks mit der Laenge "block" und werden blockweise      */

/* kopiert.                                                               */

/* - einschliesslich Mitprotokollierung der Laufzeit.                     */

/*------------------------------------------------------------------------*/



void kopieren(long anzahl,

              long blockanzahl,

              int block

              )

{

  long i,j;

  START;



  for(j = 1; j <= anzahl; j++)

  {

    if((output = fopen("schreibd.tst", "wb")) == NULL)

    {

      printf("ERROR: Kann datei nicht oeffnen\n");

      exit(1);

    }

    if((file = fopen("lesedat.tst", "rb")) == NULL)

    {

      printf("ERROR: Kann datei nicht oeffnen\n");

      exit(1);

    }

    for(i = 1; i <= blockanzahl; i++)

    {

      fread(buffer, block, 1, file);

      fwrite(buffer, block, 1, output);

    }



    fclose(output);

    fclose(file);

  }

  STOP;

  zeigeZeit();

  fclose(file);

  fclose(output);

}



/*------------------------------------------------------------------------*/

/* Prozedur zum Erstellen und Loeschen von "anzahl" Dateien der Laenge    */

/* "size".                                                                */

/* - einschliesslich Mitprotokollierung der Laufzeit.                     */

/*------------------------------------------------------------------------*/



void loeschen(long anzahl, long size)

{

  long i,j;



  START;



  for(i = 1; i <= anzahl; i++)

  {

    if((file = fopen("loeschda.tst", "wb")) == NULL)

    {

      printf("ERROR: Kann datei nicht oeffnen\n");

      exit(1);

    }

    printf("\r%3ld",i);

    for (j=0;j<size/ONE_KB;j++)

      if (fwrite (buffer, ONE_KB, 1, file)!=1)

        break;

    fclose(file);

    remove("loeschda.tst");

  }

  STOP;

  printf("\n");

  zeigeZeit();

}



/*------------------------------------------------------------------------*/

/* Prozedur zum Oeffnen und Schliessen von "anzahl" Dateien.              */

/* - einschliesslich Mitprotokollierung der Laufzeit.                     */

/*------------------------------------------------------------------------*/



void oeffnen_schliessen(long anzahl)

{

  long i;



  START;

  for(i = 1; i <= anzahl; i++)

  {

    if((file = fopen("oeffnend.tst", "r+b")) == NULL)

    {

      printf("ERROR: Kann datei nicht oeffnen\n");

      exit(1);

    }

    fclose(file);

  }

  STOP;

  zeigeZeit();

}



/*------------------------------------------------------------------------*/

/* Prozedur, die "anzahl" Suchzugriffe auf eine Datei der Laenge "anzahl" */

/* - einschliesslich Mitprotokollierung der Laufzeit.                     */

/*------------------------------------------------------------------------*/



void suchen(long anzahl, long laenge)

{

  long i;



  START;

  if((file = fopen("suchdat.tst" , "rb")) == NULL)

  {

    printf("ERROR: Kann datei nicht oeffnen\n");

    exit(1);

  }



  for(i = 1; i <= anzahl; i++)

  {

    fseek(file, (long)(rand())*(long)(rand())%laenge, SEEK_SET);

  }

  STOP;

  zeigeZeit();

  fclose(file);

}



/*------------------------------------------------------------------------*/

/* Prozedur die "anzahl" Locks auf eine Datei ab den Offset "offset" mit  */

/* der Laenge "length".                                                   */

/* - einschliesslich Mitprotokollierung der Laufzeit.                     */

/*------------------------------------------------------------------------*/



void lock_unlock(long anzahl,long offset, long length)

{

  long i,l;

  int handle;



  handle = open("lockdat.tst" , O_RDWR,0);

  START;



  for(i = 1; i <= anzahl; i++)

  {

    l = (long)(rand())*(long)(rand())%offset;

    lock(handle, l, length);

    unlock(handle, l, length);

  }

  STOP;

  zeigeZeit();



  close(handle);

}



/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

/*                                                                        */

/*                          M     A      I      N                         */

/*                                                                        */

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/



void main(int argc,char *argv[])

{



   if (argc<3)

     {

     printf("Verwende %s Testverzeichnis Protokolldatei (in Testverzeichnis)",argv[0]);

     return;

     }

#ifdef NLM

SetAutoScreenDestructionMode(TRUE);

AtUnload(flushall);

#endif

   if (chdir(argv[1]))

     {

     printf("Verzeichnis %s existiert nicht\n",argv[1]);

     return;

     }



   if ((logfile=fopen(argv[2],"wt"))==NULL)

     {

     printf("Fehler beim Erstellen der Protokolldatei\n");

     return;

     }



   log("\t\t\t\tNETZTEST\n");



   erzeuge_datei(1000, "lesedat.tst");

   log("Datei mit jeweils 10.240 Bytes in 1 Byte Schritten kopieren.\n");

   kopieren(1, 10240, 1);

   log("Datei mit jeweils 51.200 Bytes in 256 Byte Schritten kopieren.\n");

   kopieren(1, 200, 256);

   log("Datei mit jeweils 51.200 Bytes in 512 Byte Schritten kopieren.\n");

   kopieren(1, 100, 512);

   log("Datei mit jeweils 102.400 Bytes in 1.024 Byte Schritten kopieren.\n");

   kopieren(1, 100, ONE_KB);

   log("2 Dateien mit jeweils 102.400 Bytes in 10.240 Byte Schritten kopieren.\n");

   kopieren(2, 10, 10240);



   log("50 Dateien mit 1.024 Bytes erzeugen und loeschen\n");

   loeschen(50, ONE_KB);

   log("50 Dateien mit 10.240 Bytes erzeugen und loeschen\n");

   loeschen(50, 10240);

   log("50 Dateien mit 1.024.000 Bytes erzeugen und loeschen\n");

   loeschen(50, 1024000l);



   erzeuge_datei(1, "oeffnend.tst");

   log("500 Dateien oeffnen und schliessen\n");

   oeffnen_schliessen(500);



   erzeuge_datei(100, "suchdat.tst");

   log("5.000 Suchzugriffe auf eine Datei mit 102.400 Bytes\n");

   suchen(5000, 102400l);



   erzeuge_datei(100, "lockdat.tst");

   log("1.000 Lock/Unlock auf eine Datei mit 102.400 Bytes\n");

   lock_unlock(1000, 100000l, 2400);

   unlink("lesedat.tst");

   unlink("schreibd.tst");

   unlink("lockdat.tst");

   unlink("suchdat.tst");

   unlink("oeffnend.tst");

fclose(logfile);

}



