/* QM = Queue Manager, Programm zum Bearbeiten von Druckerqueues.

   Es knnen folgende Operationen ausgefhrt werden:

   - Status der Queue anzeigen bzw. verndern

   - Druckjobs anzeigen

   - Druckjobs lschen

   - Druckjobs anhalten und starten

*/



#include <stdio.h>

#include <string.h>

#include <conio.h>

#include <ctype.h>

#include <nwqueue.h>

#include <nwbindry.h>

#include <niterror.h>

#include <nwdir.h>



#define maxJobs 250

#define maxServers 25



BYTE    killJobs = 0,

        enableQueue = 0,

        disableQueue = 0,

        queueOperator =0,

        holdJobs=0,

        startJobs=0,

        noPrompt=0,

        jobOpen=0;



char    server[80],

        myName[48],queueSpec[48];



long    myID,clientIDNumber;



JobStruct job;



void    printJobInfo(long queueID,WORD jobNumber)



{

char    userName[48];

WORD    oType;

long    fileSize;

BYTE    *b;



jobOpen=0;

printf(" Job %d",jobNumber);

if (ReadQueueJobEntry(queueID,jobNumber,&job))

  printf(":\n");

else

  {

  b=(BYTE *) job.jobEntryTime;

  clientIDNumber=job.clientIDNumber;

  printf(" [%3d]:",job.jobPosition);

  if (! GetBinderyObjectName(job.clientIDNumber,userName,&oType))

    {

    printf(" User %s, ",userName);

    printf("%d.%d.%02d %d:%02d.%02d ",b[2],b[1],b[0],b[3],b[4],b[5]);

    }

  if (job.serverIDNumber != NULL)

    printf("[active] ");

  else

    if (! (job.jobControlFlags & ~0x1f) )

      printf("[ready] ");

  if (job.jobControlFlags & QF_ENTRY_OPEN)

    {

    printf("[adding] ");

    jobOpen = 1;

    }

  if (job.jobControlFlags & QF_USER_HOLD)

    printf("[user hold] ");

  if (job.jobControlFlags & QF_OPERATOR_HOLD)

    printf("[operator hold] ");

  if (! GetQueueJobsFileSize(queueID,jobNumber,&fileSize))

    printf("\n   (%s) %s byte(s)\n",job.textJobDescription,ltoa(fileSize,userName,10));

  else

    putchar('\n');

  }

}



void explain(char *progName)

{

printf("Usage %s ? | [[server/]queuespec [option] [NOP[ROMPT]\n",progName);

printf("  ? : get this explanation \n");

printf("  server : name of file server, default server if omitted\n");

printf("  queuespec : name of queue, wildcards allowed, default = \"*\"\n");

printf("  option : one of the following\n");

printf("    KIL[L]    : kill printjobs for specified queue(s)\n");

printf("    ENA[BLE]  : enable specified queue(s) \n");

printf("    DIS[ABLE] : disable specified queue(s) \n");

printf("    HOL[D]    : hold printjobs for specified queue(s)\n");

printf("    STA[RT]   : start printjobs for specified queue(s)\n");

printf("  NOP[ROMPT]  : suppress prompting\n");

exit(0);

}



void parseCommandLine(int argc,char *argv[])

{

char    *p,*fname,*help,*queuespec;



if ((p=strrchr(argv[0],'.'))!=NULL)

  *p=0;

if ( (p=strrchr(argv[0],'\\'))!=NULL)

  fname=p+1;

else

  fname=argv[0];

if (!strcmp(argv[1],"?"))

  explain(fname);

switch (argc)

  {

  case 1 :

    strcpy(queueSpec,"*");

    break ;

  default :

    if (help=strchr(argv[1],'/'))

      {

      queuespec = strupr(help+1);

      *help=0;

      strcpy(server,strupr(argv[1]));

      }

    else

      queuespec = strupr(argv[1]);

    strcpy(queueSpec,queuespec);     /* given pattern */

    if (argc > 2)

      {

      killJobs = (!strnicmp(argv[2],"KIL",3));

      disableQueue = (!strnicmp(argv[2],"DIS",3));

      enableQueue = (!strnicmp(argv[2],"ENA",3));

      holdJobs = (!strnicmp(argv[2],"HOL",3));

      startJobs = (!strnicmp(argv[2],"STA",3));

      if (argc>3)

        noPrompt = (!strnicmp(argv[3],"NOP",3));

      }

   }

}



int  Yes(char *prompt)

{

char c;

if (noPrompt)

  return(1);

printf("%s ",prompt);

c=toupper(getche());

if (c=='Q')

  exit(0);

return(c=='Y');

}



void processQueue(char *queueName,long queueID)

{

long    serverIDList[maxServers];



WORD    jobCount,

        jobList[maxJobs],

        serverStationList[maxServers];



BYTE    queueStatus,numberOfJobs,numberOfServers;



int     cC,i;



cC = ReadQueueCurrentStatus(queueID,&queueStatus,&numberOfJobs,

     &numberOfServers,serverIDList,serverStationList,maxServers);



if (! cC)

  {

  queueOperator=!SetQueueCurrentStatus(queueID,queueStatus);

  printf("%s/%-16s ",server,queueName);

  if (!queueStatus)

    printf("[available]\n");

  else

    {

    if (queueStatus & QS_CANT_ADD_JOBS)

      printf("[adding disabled] ");

    if (queueStatus & QS_SERVERS_CANT_ATTACH)

      printf("[no attach] ");

    if (queueStatus & QS_CANT_SERVICE_JOBS)

      printf("[no service]");

    putchar('\n');

    }

  if (disableQueue && !(queueStatus & QS_CANT_ADD_JOBS))

    {

    if (Yes("disable queue (Y/N/Q)? "))

      if (!SetQueueCurrentStatus(queueID,queueStatus | QS_CANT_ADD_JOBS))

        puts(" disabled");

      else

        puts(" error changing queue status");

    else

      puts(" status unchanged");

    return;

    }

  if (enableQueue && (queueStatus & QS_CANT_ADD_JOBS))

    {

    if (Yes("enable queue (Y/N/Q)? "))

        if (!SetQueueCurrentStatus(queueID,queueStatus & ~QS_CANT_ADD_JOBS))

          puts(" enabled");

        else

          puts(" error changing queue status");

      else

        puts(" status unchanged");

    return;

    }



    if (! GetQueueJobList(queueID,&jobCount,jobList,maxJobs))

      for (i=0; i<numberOfJobs; i++)

        {

        printJobInfo(queueID,jobList[i]);

        if (killJobs && (!jobOpen) && (queueOperator || myID==clientIDNumber) )

          {

          if (Yes("remove job (Y/N/Q)? "))

            if (!RemoveJobFromQueue(queueID,jobList[i]))

              puts(" removed");

            else

              puts(" error removing job");

          else

            puts(" not removed");

          }

        if (holdJobs && (!jobOpen) && (queueOperator || myID==clientIDNumber) )

          {

          if (Yes("hold job (Y/N/Q)? "))

            {

            job.jobControlFlags|=QF_USER_HOLD;

            if (!ChangeQueueJobEntry(queueID,&job))

              puts(" hold");

            else

              puts(" error holding job");

            }

          else

            puts(" not changed");

          }

        if (startJobs && (!jobOpen) && (queueOperator || myID==clientIDNumber) )

          {

          if (Yes("start job (Y/N/Q)? "))

            {

            job.jobControlFlags&=~QF_USER_HOLD;

            if (!ChangeQueueJobEntry(queueID,&job))

              puts(" started");

            else

              puts(" error starting job");

            }

          else

            puts(" not changed");

          }

        if (i == numberOfJobs-1)

          putchar('\n');

        }

  }

}



void main(int argc,char **argv)

{

int     cC,sCC;



long    queueID;



char    queueName[100],

        objectHasProperties,

        objectFlag,

        objectSecurity;



WORD    objectType,

        searchObjectType,

        connectionID;



BYTE    d;



parseCommandLine(argc,argv);

GetFileServerName(0,server);



/* if (cC = GetConnectionID(server,&connectionID))

  {

  if (cC==UNKNOWN_FILE_SERVER)

    printf("Unknown server %s\n",server);

  else

    printf("You are not attached to server %s\n",server);

  exit(1);

  }

SetPreferredConnectionID(connectionID);

*/

if (!GetBinderyAccessLevel(&d,&myID))

  GetBinderyObjectName(myID,myName,&objectType);

else

  return;

printf("User %s (ID %lx) on file server %s\n",myName,myID,server);



searchObjectType = OT_PRINT_QUEUE;      /* search for print queues */



queueID = -1l;                          /* value for first call */

sCC = 0;

while (! sCC)

  {

  sCC = ScanBinderyObject(queueSpec,searchObjectType,&queueID,

                         queueName,&objectType,&objectHasProperties,

                         &objectFlag,&objectSecurity);



  if (! sCC )   /* object found */

    processQueue(queueName,queueID);

  }

}



