/*

   Enhancment for Radius Authentication Server:

   Hooks NWDSVerifyObjectPasswort to add the following functions:

   - only usernames matching the 3 letter 5 digit scheme are allowed to

     authenticate

   - members of exclude group are rejected



*/



#include <stdlib.h>

#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <string.h>

#include <ctype.h>

#include <io.h>



#include <library.h>

#include <process.h>

#include <nwenvrn.h>

#include <advanced.h>

#include <nwqueue.h>

#include <nwmsg.h>

#include <niterror.h>

#include <nwdsapi.h>

#include <nwcntask.h>

#include <nwsnut.h>

#include <nwmisc.h>

#include <time.h>



#define MYINI "SYS:SYSTEM\\RADENH.INI"

#define PLINES 8



NUTInfo *handle;

LONG    NLMHandle;

LONG    allocTag;

int     CLIBScreenID,escapeMode,inForm=0,pLine=0;

PCB     *myPortal;



enum { MSG_NAME, MSG_VER, MSG_LIST, MSG_FORM };

char    *messages[16]={ "RADENH",

                        "1.01 (c) 1997 Peter Heitzer",

                        "RADENH options",

                        "RADENH options"

                        };

char gtmp[256];  // global temp buffer

NWDSContextHandle   dContext;



LONG libHandle=0;

int  maxLen=0,queueCnt=0,run=0;



/* patch options */



int patchActive=1, excludeOption=0;

char excludeList[256];



int clientCleanupFunction(void *p)

{

return 0;

}



/* cleanup function:

   deregisters library and restores nut */



void cleanup(void)

{

if (libHandle)

  DeregisterLibrary(libHandle);

if (handle)

  NWSRestoreNut(handle);

if (CLIBScreenID)

  DestroyScreen(CLIBScreenID);

NWDSFreeContext(dContext);

}



int readParams(void)

{

FILE *f;

char line[256],*q,*p1;



if ((f=fopen(MYINI,"r"))==NULL)

  return 1;



do {

   if (fgets(line,256,f)==NULL)

     break;

   if (*line==';')

     continue;

   if (q=strtok(line,","))

     if (p1=strtok(NULL,",\n\r"))

       {

       if (!stricmp(q,"Exclude"))

         strcpy(excludeList,p1);

       }

   } while (1);

fclose(f);

return 0;

}



/* check if good username, i.e. 3 characters and 5 digits as in abc12345

   returns 0 if ok 1 else

*/



int verifyName(char *uname)

{

int i;

for (i=0;i<3;i++)

  if (!isalpha(uname[i]))

    return 1;

for (i=3;i<8;i++)

  if (!isdigit(uname[i]))

    return 1;

return 0;

}



/* convert name to RFC syntax, i.e. cn=xx.ou=yy -> xx.yy */



char *toRFC(char *s)

{

static char rfc[256];

       char tmp[256],*p,*peq;

int    first;



*rfc=0;



if (*s=='.') // absolute name, keep dot

  {

  strcpy(rfc,".");

  strcpy(tmp,s+1);

  }

else

  strcpy(tmp,s);



for (first=1,p=strtok(tmp,".");p;p=strtok(NULL,"."))

  {

  if (!first)

    strcat(rfc,".");

  else

    first=0;

  if (peq=strchr(p,'='))

    strcat(rfc,peq+1);

  else

    strcat(rfc,p);

  }

return strlwr(rfc);

}



void portalPrint(char *s)

{

if (pLine<PLINES)

  {

  NWSDisplayTextInPortal(pLine,0,s,VNORMAL,myPortal);

  pLine++;

  }

else

  {

  NWSScrollPortalZone(0,0,PLINES,78,0,1,V_UP,myPortal);

  NWSDisplayTextInPortal(PLINES-1,0,s,VNORMAL,myPortal);

  }

if (!inForm)

  NWSUpdatePortal(myPortal);

}



/*

hook for NWDSVerifyObjectPassword

*/



int nWDSVerifyObjectPassword (

    NWDSContextHandle   contextHandle,

    LONG               optionsFlag,

    char *             objectName,

    char *              password)



{

int cC;

char tmp[256];

char *uname=toRFC(objectName);



if (!patchActive)

  return NWDSVerifyObjectPassword (contextHandle,optionsFlag,

  objectName,password);



if (verifyName(uname))

  cC=0xfffffda7;

else

  {

  cC=NWDSLogin (contextHandle,optionsFlag,objectName,password,0);

  if (cC==0)

    NWDSLogout (contextHandle);

  }

// NWSClearPortal(myPortal);

sprintf(tmp,"%s -> %x\n",uname,cC);

// NWSDisplayTextInPortal(0,0,tmp,VNORMAL,myPortal);

portalPrint(tmp);



// NWSUpdatePortal(myPortal);

return cC;

}



int ListAction (LONG keyPressed, LIST **elementSelected, LONG *itemNumber, void *listParameter)

{

char tmp[16];



elementSelected = elementSelected;

listParameter = listParameter;



if (keyPressed == M_ESCAPE)

  return 1;



NWSPushList(handle);

NWSInitForm(handle);



NWSAppendCommentField (1, 1, "Patch active: ", handle);

NWSAppendBoolField (1, 20, NORMAL_FIELD, &patchActive, NULL, handle);

NWSAppendCommentField (2, 1, "Exclude option: ", handle);

NWSAppendBoolField (2, 20, NORMAL_FIELD, &excludeOption, NULL, handle);

NWSAppendCommentField (3, 1, "Exclude list: ", handle);

NWSAppendCommentField (3, 20, excludeList, handle);



inForm=1;

NWSEditPortalForm (MSG_FORM, 5, 40, 5, 70, F_NOVERIFY, F_NO_HELP, 3, handle);

NWSDestroyForm (handle);

NWSPopList(handle);

inForm=0; NWSUpdatePortal(myPortal);

return -1;

}



void main(int argc,char *argv[])

{

int     i;

LONG    ccode, infoPortalNumber;

PCB     *ipPtr;

WORD    screenHeight, screenWidth;



if (readParams())

  {

  ConsolePrintf("Error reading %s\n",MYINI);

  return;

  }



dContext = NWDSCreateContext( );



if (dContext==ERR_CONTEXT_CREATION)

  {

  ConsolePrintf("Error creating context\n");

  return;

  }





NLMHandle=GetNLMHandle();



CLIBScreenID = CreateScreen("RADENH Screen", AUTO_DESTROY_SCREEN);

if (!CLIBScreenID)

  return;

allocTag = AllocateResourceTag(NLMHandle, "RADENH Alloc Tag",AllocSignature);



if (!allocTag)

  {

  DestroyScreen(CLIBScreenID);

  return;

  }



if (NWSInitializeNut(MSG_NAME,MSG_VER,NORMAL_HEADER,NUT_REVISION_LEVEL,

    messages, 0, CLIBScreenID, allocTag, &handle))

        {

        DestroyScreen(CLIBScreenID);

        return;

        }



AtUnload(cleanup);



DisplayScreen(CLIBScreenID);

GetSizeOfScreen (&screenHeight, &screenWidth);



infoPortalNumber = NWSCreatePortal(14l,0l,PLINES+2,(LONG)screenWidth,

                        9l,(LONG)screenWidth-2, SAVE,

                        NULL,/* "RADENH Status",*/

                        VNORMAL, SINGLE, VNORMAL, CURSOR_OFF, VIRTUAL, handle);



if (infoPortalNumber > MAXPORTALS)

  {

  printf ("Unable to create information portal\n");

  DestroyScreen(CLIBScreenID);

  return;

  }



NWSGetPCB (&ipPtr, infoPortalNumber, handle);

myPortal=ipPtr;

NWSClearPortal(myPortal);

NWSUpdatePortal(myPortal);



if ((libHandle=RegisterLibrary(clientCleanupFunction))!=-1)

  {

  ConsolePrintf("Library registered\n");

  run=1;

  AtUnload(cleanup);

  }



NWSInitList (handle, free);



NWSAppendToList ("Press enter to modify options",(void *) 0, handle);



NWSList (MSG_LIST, 5, 40, 1, 40,

                M_NO_SORT | M_SELECT , NULL, handle, NULL, ListAction, 0);



NWSDestroyList (handle);



while (run) delay(1000);



NWSDestroyPortal (infoPortalNumber, handle);

cleanup();

}



