/*

  simple scheduler that starts NLMs at periodic times

*/



#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <process.h>

#include <time.h>

#include <string.h>

#include <ctype.h>

#include <stdlib.h>



#define MAX_NLMS 10         /* maximum of NLMs to be scheduled */

#define NR_OF_KEYWORDS 13   /* maximum of supported keywords */

#define VERSION "Netware NLM Scheduler 1.0"



#define ES_OPEN 1

#define ES_READ 2

#define ES_SYNTAX 3

#define ES_NONLM 4



#define ONE_MIN 60

#define ONE_HOUR 3600

#define ONE_DAY 86400

#define ONE_WEEK ONE_DAY*7



#define DELIM "\n\r\t "



typedef enum { S_MULTI, S_MIN, S_HOUR, S_DAY, S_WEEK, S_WEEKDAY, S_ONCE } SCHEDULE_TYPE;



typedef struct {

               char keyword[10];

               SCHEDULE_TYPE scheduleType;

               time_t value;

               } KEYWORD;



KEYWORD keywords[NR_OF_KEYWORDS] = {

               "EVERY",S_MULTI,-1,

               "MINUTE",S_MIN,ONE_MIN,

               "HOUR",S_HOUR,ONE_HOUR,

               "DAY",S_DAY,ONE_DAY,

               "WEEK",S_WEEK,ONE_WEEK,

               "SUNDAY",S_WEEKDAY,ONE_WEEK,

               "MONDAY",S_WEEKDAY,ONE_WEEK,

               "TUESDAY",S_WEEKDAY,ONE_WEEK,

               "WEDNESDAY",S_WEEKDAY,ONE_WEEK,

               "THURSDAY",S_WEEKDAY,ONE_WEEK,

               "FRIDAY",S_WEEKDAY,ONE_WEEK,

               "SATURDAY",S_WEEKDAY,ONE_WEEK,

               "AT",S_ONCE,-1 };



typedef struct {

               SCHEDULE_TYPE intervalType; /* type of schedule interval */

               char NLMName[64];           /* name of NLM */

               char NLMCommand[128];       /* commandline for NLM */

               time_t nextDate;            /* next start date */

               time_t interval;            /* time between events */

               } SCHEDULED_OBJECT;



int nrOfNLMs=0;

SCHEDULED_OBJECT scheduledNLM[MAX_NLMS];



void DisplaySchedule(void)

{

int i;

SCHEDULED_OBJECT *sO;



char *st;

if (!nrOfNLMs)

  return;

else

  printf("Nr: %-16s %-16s %9s %s","NLMName","NLMCommand","Interval ","Next execution time\n");



for (i=0,sO=scheduledNLM;i<nrOfNLMs;i++,sO++)

  {

  st=ctime(&sO->nextDate);

  printf("%2d: %-16s %-16s %8lds %s",i+1,sO->NLMName,sO->NLMCommand,sO->interval,st);

  }

}



/* return pointer to keyword structure or NULL if keyw is no valid keyword */



KEYWORD *keyword(char *keyw)

{

int i;

if (keyw==NULL)

  return(NULL);

for (i=0;i<NR_OF_KEYWORDS;i++)

  if (!strnicmp(keywords[i].keyword,keyw,strlen(keywords[i].keyword)))

    return(&keywords[i]);

return(NULL);

}



int AddScheduledObject(char *line)

{

char *p;

int cC=0,n;

KEYWORD *keyw;

SCHEDULED_OBJECT *sNLM;

time_t curTime;

struct tm *t;

if (nrOfNLMs==MAX_NLMS)

  return(-1);

do

  {

  p=strtok(line,DELIM);

  if (p==NULL)

    break;

  if ((keyw=keyword(p))==NULL)

    {

    cC=ES_SYNTAX;

    break;

    }

  sNLM=&scheduledNLM[nrOfNLMs];

  sNLM->nextDate=time(NULL);

  *sNLM->NLMName=0;

  *sNLM->NLMCommand=0;

  if (keyw->value==-1)

    {

    if (keyw->scheduleType==S_ONCE)

      {

      sNLM->intervalType=S_ONCE;

      if ((p=strtok(NULL,DELIM))==NULL)

        cC=ES_SYNTAX;

      }

    if (keyw->scheduleType==S_MULTI)

      {

      sNLM->intervalType=S_MULTI;

      if ((p=strtok(NULL,DELIM))==NULL)

        {

        cC=ES_SYNTAX;

        break;

        }

      n=1;

      if (isdigit(*p))

        {

        n=atoi(p);

        p=strtok(NULL,DELIM);

        }

      if ((keyw=keyword(p))==NULL)

        {

        cC=ES_SYNTAX;

        break;

        }

        if (keyw->value==-1) /* no interval given */

          {

          cC=ES_SYNTAX;

          break;

          }

      sNLM->intervalType=keyw->scheduleType;

      sNLM->interval=n*keyw->value;

      sNLM->nextDate+=sNLM->interval;

      }

    if ((p=strtok(NULL,DELIM))!=NULL)

      {

      strcpy(sNLM->NLMName,p);

      if ((p=strtok(NULL,"\n\r"))!=NULL)

        strcpy(sNLM->NLMCommand,p);

      }

    else

      cC=ES_NONLM;

    }

  else

    cC=ES_SYNTAX;

  if (cC)

    break;

  nrOfNLMs++;

  } while (FALSE);

return(cC);

}



/* read schedule file */



int ReadScheduleFile(char *name)

{

char line[256];

FILE *scheduleFile;

int cC=0;



if ((scheduleFile=fopen(name,"rt"))==NULL)

  return(ES_OPEN);



while (!feof(scheduleFile))

  {

  if (nrOfNLMs==MAX_NLMS)

    break;

  if (fgets(line,sizeof(line),scheduleFile)==NULL)

    break;

  if (cC=AddScheduledObject(line))

    break;

  }

fclose(scheduleFile);

return(cC);

}



void main(int argc,char *argv[])

{

int cC,n;

char command[128],*c;



if (argc==1)

  {

  printf("Usage %s filename\n",argv[0]);

  return;

  }

if (cC=ReadScheduleFile(argv[1]))

  {

  switch (cC) {

    case ES_OPEN : printf("Error opening %s\n",argv[1]); break;

    case ES_SYNTAX : printf("Syntax error in line %d\n",nrOfNLMs+1); break;

    case ES_NONLM : printf("Missing target in line %d\n",nrOfNLMs+1); break;

    default : printf("Error %d\n",cC);

    }

  return;

  }

do

  {

  *command=0;

  printf("-> ");

  fgets(command,sizeof(command),stdin);

  if (*command)

    {

    strupr(command);

    c=strtok(command," \n\t\r");

    if (!strcmp(c,"EXIT"))

      return;

    if (!strcmp(c,"VER"))

      printf("%s\n",VERSION);

    if (!strcmp(c,"SHOW"))

      DisplaySchedule();

    if (!strcmp(c,"ADD"))

      {

      cC=AddScheduledObject(fgets(command,sizeof(command),stdin));

      if (!cC)

        DisplaySchedule();

      else

        printf("Error %d\n",cC);

      }

    if (!strcmp(c,"DEL"))

      {

      printf("Enter number: ");

      fscanf(stdin,"%d%c",&n,command);

      if ((!n)||(n>nrOfNLMs))

        printf("Invalid number\n");

      else

        {

        memcpy(&scheduledNLM[n-1],&scheduledNLM[--nrOfNLMs],sizeof(SCHEDULED_OBJECT));

        DisplaySchedule();

        }

      }

    if (!strcmp(c,"HELP"))

      printf("Supported commands are ADD DEL EXIT HELP SHOW VER\n");

    }

  } while(TRUE);

}

