/*

This NLM patches two CLIB calls to enable forms for ATPS.NLM

   1. ServiceQueueJobAndOpenFile

      If patch is active parameter targetJobType (normally 0xffff) will be

      replaced with the wanted form number.

   2. SubmitAccountCharge

      If patch is active parameter chargeAmount will newly be calculated for

      the given form number.

   The amounts will be taken out of ATPS.CFG

*/



#include <stdlib.h>

#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <string.h>

#include <ctype.h>



#include <library.h>

#include <process.h>

#include <nwenvrn.h>

#include <advanced.h>

#include <nwqueue.h>

#include <nwaccntg.h>

#include <nwbindry.h>



#define MAXFORMS 8

#define ATPS_CFG "SYS:SYSTEM\\ATPS.CFG"



typedef enum { C_ON, C_OFF, C_FORM, C_STAT, C_EXIT, C_NULL, C_HELP } COMMAND_TYPE;



typedef struct {

               char name[8];

               COMMAND_TYPE type;

               } COMMAND;



typedef struct {

               char name[48];

               int number;

               long accountPoints;

               long queueID;

               } FORMTYPE;



COMMAND commands[]= {

        "ON",C_ON,

        "OFF",C_OFF,

        "FORM",C_FORM,

        "STAT",C_STAT,

        "EXIT",C_EXIT,

        "HELP",C_HELP,

        NULL,C_NULL };



FORMTYPE forms[MAXFORMS],*currentForm=NULL;



LONG libHandle=0,clientID=0,clientQueueID=0;

char myName[48],formName[256]="";

int  formNum=0,run=0,theConsole=0,patchCall=0,formCount,

     numCommands=sizeof(commands)/sizeof(COMMAND_TYPE);

long defaultAccountPoints=1;



LONG myCommand(LONG screenID,BYTE *cmdline);



struct commandParserStructure

  myParserStruct={

    0,

    myCommand,

    0 };



FORMTYPE *getFormType(int formnum)

{

int i;

FORMTYPE *f;

for (i=0,f=NULL;i<formCount;i++)

  if (forms[i].number==formnum)

    return &forms[i];

return f;

}



long getAccountPointsForQueue(long queueID)

{

int i;

long a;

for (i=0,a=defaultAccountPoints;i<formCount;i++)

  if (forms[i].queueID==queueID)

    return forms[i].accountPoints;

return a;

}



COMMAND_TYPE getCommandType(char *cmd)

{

int i;

COMMAND_TYPE c;

if (cmd==NULL)

  return C_NULL;

for (i=0,c=C_NULL;i<numCommands;i++)

  if (!stricmp(cmd,commands[i].name))

    return commands[i].type;

return c;

}



int clientCleanupFunction(void *p)

{

return 0;

}



/* cleanup function:

   deregisters library */



void cleanup(void)

{

if (libHandle)

   DeregisterLibrary(libHandle);

}



/* remove CC Handler */



void removeCCHandler(void)

{

UnRegisterConsoleCommand(&myParserStruct);

}



void doStat(void)

{

FORMTYPE *f=getFormType(formNum);

if (f)

  strcpy(formName,f->name);

ConsolePrintf("SF is %s, form is %d (%s), %d points\n\r",

  patchCall ? "ON" : "OFF",formNum,formName,

  currentForm ? currentForm->accountPoints : defaultAccountPoints);

if (clientID)

  ConsolePrintf("Client ID: %x queue ID %x\n,\r",clientID,clientQueueID);

}



void doHelp(void)

{

ConsolePrintf("Command syntax: SF <command> [parameter]\n\r");

ConsolePrintf("Supported commands are:\n\r");

ConsolePrintf("  EXIT : unload SETATPSF\n\r");

ConsolePrintf("  FORM <number> : set active form to <number>\n\r");

ConsolePrintf("  HELP : show this help\n\r");

ConsolePrintf("  OFF : disable forms\n\r");

ConsolePrintf("  ON : enable forms\n\r");

ConsolePrintf("  STAT : show status\n\r");

}



/* my console command routine */



LONG myCommand(LONG screenID,BYTE *cmdline)

{

char *cmd,*f;

int i;

FORMTYPE *form;



if (screenID==theConsole)

  {

  if (!strnicmp(cmdline,"SF ",3))

    {

    cmd=strtok(cmdline+3," \n\r\t");

    switch (getCommandType(cmd)) {

      case C_EXIT: run=0; return 0;

      case C_HELP: doHelp(); return 0;

      case C_STAT: doStat(); return 0;

      case C_ON:  patchCall=1; doStat(); return 0;

      case C_OFF:  patchCall=0; doStat(); return 0;

      case C_FORM:

        if (f=strtok(NULL," \n\r\t"))

          {

          if (isdigit(*f))

            {

            if (form=getFormType(atoi(f)))

              {

              currentForm=form;

              formNum=form->number;

              strcpy(formName,form->name);

              }

            else

              ConsolePrintf("Invalid form number\n\r");

            }

          else

            ConsolePrintf("Invalid form number\n\r");

          doStat();

          }

        else

          ConsolePrintf("Missing form number\n\r");

        return 0;

      default: ConsolePrintf("Invalid command for SF\n\r");

               doHelp();

               return 0;

      } // case

    }

  }

else

  return 1;

}



int   serviceQueueJobAndOpenFile(

         long __queueID,

         WORD __targetJobType,

         JobStruct *__job,

         int *__fileHandle )



{

int cC;

if (patchCall)

  __targetJobType=formNum;

cC=ServiceQueueJobAndOpenFile(

         __queueID,__targetJobType,__job,__fileHandle );



if (!cC)

  {

  clientID=__job->clientIDNumber;

  clientQueueID=__queueID;

  }

return cC;

}



int   submitAccountCharge(

          WORD __binderyObjectType,

          char *__binderyObjectName,

          WORD __serviceType,

          long __chargeAmount,

          long __cancelHoldAmount,

          WORD __commentType,

          char *__comment )



{

long ID;

if (patchCall)

  {

  GetBinderyObjectID(__binderyObjectName,__binderyObjectType,&ID);

  if (ID)

    if (ID==clientID)

      {

      ConsolePrintf("Charging %s %d\n\r",__binderyObjectName,__chargeAmount);

      }

  }

return   SubmitAccountCharge(

          __binderyObjectType,

          __binderyObjectName,

          __serviceType,

          __chargeAmount,

          __cancelHoldAmount,

          __commentType,

          __comment );

}



/* read ATPS.CFG and set forms[] */



void readATPS_CFG(void)

{

FILE *f;

static char line[256],tmp[64];

char *p,*q1,*q2;



formCount=0;

if ((f=fopen(ATPS_CFG,"r"))==NULL)

  return;

while (!feof(f))

  {

  if (formCount==MAXFORMS)

    break;

  if (fgets(line,255,f)==NULL)

    break;

  if (strlen(line)<3)

    break;

  forms[formCount].number=0;

  sprintf(forms[formCount].name,"Form %d",formCount);

  if (p=strstr(line,"-m"))

    forms[formCount].number=atoi(p+2);

  if (p=strstr(line,"-u"))

    forms[formCount].accountPoints=atol(p+2);

  if (p=strstr(line,"-q"))

    {

    p=strtok(strncpy(tmp,p+2,48)," \n\t\r");

    GetBinderyObjectID(p,OT_PRINT_QUEUE,&forms[formCount].queueID);

    }

  if (p=strstr(line,"-o"))

    if (q1=strchr(p+1,'\"'))

      if (q2=strchr(q1+1,'\"'))

        {

        *q2=0;

        strcpy(forms[formCount].name,q1+1);

        }

  ConsolePrintf("Form %d '%s' %d (q=%x).\n\r",forms[formCount].number,

    forms[formCount].name,forms[formCount].accountPoints,forms[formCount].queueID);

  formCount++;

  }

fclose(f);

}



void main(void)

{

LONG consoleCommandTag;

unsigned myNLMHandle;

myNLMHandle=GetNLMHandle();



theConsole=__GetScreenID(CreateScreen("System Console",0));



if ((libHandle=RegisterLibrary(clientCleanupFunction))!=-1)

  {

  if (consoleCommandTag=AllocateResourceTag(myNLMHandle,

    "SETATPSF console command",ConsoleCommandSignature))

    {

    myParserStruct.RTag=(struct ResourceTagStructure *)consoleCommandTag;

    RegisterConsoleCommand(&myParserStruct);

    atexit(removeCCHandler);

    run=1;

    patchCall=1;

    readATPS_CFG();

    if (currentForm=getFormType(0))

      defaultAccountPoints=currentForm->accountPoints;

    else

      {

      ConsolePrintf("No valid entries in %s found. SETATPSF not loaded.\n\r",

        ATPS_CFG);

      exit(0);

      }

    ConsolePrintf("SETATPSF loaded. Type SF HELP to get command list\n\r");

    }

  atexit(cleanup);

  ExitThread(TSR_THREAD,0);

  }

}

