/* definitions for standard programs */



#include <stdlib.h>

#include <conio.h>

#include <stdio.h>

#include <errno.h>

#include <string.h>

#include <ctype.h>

#include <io.h>



#include <library.h>

#include <process.h>

#include <nwenvrn.h>

#include <advanced.h>

#include <nwqueue.h>

#include <nwmsg.h>

#include <niterror.h>

#include <nwdsapi.h>

#include <nwcntask.h>

#include <nwsnut.h>

#include <nwmisc.h>

#include <time.h>



#define MYNAME "XATPSENH "

#define MYLEN strlen(MYNAME)

#define MYINI "SYS:SYSTEM\\XATPSENH.INI"

#define MYLOG "SYS:SYSTEM\\XATPSENH.LOG"



#define MAXQ 16

#define BANNER_ON 0x8000

#define BANNER_OFF 0x7fff



NUTInfo *handle;

LONG    NLMHandle;

LONG    allocTag;

int     CLIBScreenID,escapeMode,inForm=0;

PCB     *myPortal;



enum { MSG_NAME, MSG_VER, MSG_LIST, MSG_FORM };

char    *messages[16]={ "XATPSENH",

                        "1.2 (c) 1997 Peter Heitzer",

                        "Queues managed by XATPSENH",

                        "Queue Info",

                        "Exit" };

char gtmp[256];  // global temp buffer

NWDSContextHandle   dContext;



LONG libHandle=0;

int  maxLen=0,queueCnt=0,run=0;



enum { KEEP_BANNER=0, FORCE_BANNER, NO_BANNER };



typedef struct {

               LONG queueID;        // queue ID

               BYTE filterPS;       // if true remove non PS jobs

               int bannerMode;      // banner mode: keep/force/no banner

               int callCount;       // how often patch was called

               int jobsDeleted;     // nr of removed jobs

               int patchCall;       // disable/enable patch

               char queueName[256]; // name of queue

               } QINFO;



QINFO queues[MAXQ];



/* write log file

  Parameters:  qi       : QINFO pointer, can be NULL

               name     : name of job owner or general message if qi==NULL

               fname    : job description

               dumpbuff : optional buffer to be dumped

               fsize    : job file size

*/



void writeLog(QINFO *qi,char *name,char *fname,char *dumpbuf,long fsize)

{

time_t t;

FILE *logFile;

int i,j;

char *p,tmp[1024];



t=time(NULL);



if (logFile=fopen(MYLOG,"a+"))

  {

  p=tmp;

  strftime(p,50,"%d.%m.%y %H.%M.%S",localtime(&t)); strcat(p,"\n");

  p=strchr(p,0);

  NWSClearPortal(myPortal);

  if (qi)

    {

    sprintf(p," Queue: %s\n User: %s\n Job: %s\n Size: %ld bytes\n",

    qi->queueName,name,fname,fsize);

    p=strchr(p,0);

    }

  else

    {

    sprintf(p," %s\n",name);

    p=strchr(p,0);

    }

  /* dump first 64 byte of dumpbuf */

  if (dumpbuf)

    {

    for (i=0;i<64;i+=16)

      {

      sprintf(p," %04X: ",i); p=strchr(p,0);

      for (j=0;j<16;j++)

        {

        sprintf(p,"%02X ",(BYTE) dumpbuf[i+j]);

        p=strchr(p,0);

        }

      for (j=0;j<16;j++)

        {

        sprintf(p,"%c",dumpbuf[i+j]>31 ? dumpbuf[i+j] : '.');

        p=strchr(p,0);

        }

      sprintf(p,"\n");

      p=strchr(p,0);

      }

    }

  fprintf(logFile,"%s\n",tmp);

  fclose(logFile);

  NWSDisplayTextInPortal(0,0,tmp,VNORMAL,myPortal);

  if (!inForm)

    NWSUpdatePortal(myPortal);

  }

}



LONG myGetLocalObjectID(char *oName)

{

WORD connID;

LONG oID=NULL;

int cC;



SetCurrentConnection (-1);

connID = GetCurrentConnection();

cC=NWDSMapNameToID(dContext,connID,oName,&oID);

ReturnConnection(connID);

return cC ? NULL : oID;

}



char *myMapLocalObjectIDToName(LONG ID)

{

WORD connID;

static char name[256];

int cC;



SetCurrentConnection (-1);

connID = GetCurrentConnection();

cC=NWDSMapIDToName(dContext,connID,ID,name);

ReturnConnection(connID);

return cC ? "unknown" : name;

}



/* read ini file and set parameters for every queue

   format of ini file:



   queuename,patchFlag,bannerMode,filterPS

   lines starting with ';' are comments



*/



int readParams(void)

{

FILE *f;

char line[256],*q,*p1,*p2,*p3;

QINFO *qi;



if ((f=fopen(MYINI,"r"))==NULL)

  return 1;

do {

   if (fgets(line,256,f)==NULL)

     break;

   if (*line==';')

     continue;

   if (q=strtok(line,",\n\r"))

     if (p1=strtok(NULL,","))

       if (p2=strtok(NULL,","))

         if (p3=strtok(NULL,","))

           {

           qi=&queues[queueCnt];

           qi->filterPS=atoi(p3);

           qi->bannerMode=atoi(p2);

           qi->patchCall=atoi(p1);

           strcpy(qi->queueName,q+(*q=='.' ? 1 : 0));

           if (qi->queueID=LongSwap(myGetLocalObjectID(q)))

             {

             maxLen=max(maxLen,strlen(q));

             queueCnt++;

             }

         if (queueCnt>=MAXQ)

           break;

         }

   } while (1);

fclose(f);

return 0;

}



/* get pointer to QINFO structure for queue ID or NULL if queue ID was

not found */



QINFO *findQueue(LONG queueID)

{

int i;

for (i=0;i<queueCnt;i++)

  if (queues[i].queueID==queueID)

    return &queues[i];

return NULL;

}



/* return true if job file contains postscript */



int isPSjob(int f)

{

char *p;

int isPS=1;



*gtmp=0;

if (read(f,gtmp,256)!=-1)

  {

  lseek(f,0l,SEEK_SET);

  isPS=0;

  if (p=strtok(gtmp,"\004\n\r"))

    if (!strncmp(p,"%!",2))

      isPS=1;

  }

return isPS;

}



/* remove invalid job */



int removeJob(LONG queueID,NWQJobRec_t *jobInfo,int *handle)

{

WORD station[1];

BYTE result[1];

static char tmp[128];

int cC;

long s=0;



NWQGetJobFileSize(queueID,jobInfo->job.num,&s);

if (!(cC=NWQEndJobService(queueID,jobInfo->job.num,0l,*handle)))

  {

  station[0]=jobInfo->client.clientConnNum;

  sprintf(tmp,"Non PS job (%s) removed",jobInfo->textJobDescription);

  SendBroadcastMessage(tmp,station,result,1);

  sprintf(tmp,"%lx (%s)",jobInfo->job.num,jobInfo->textJobDescription);

  writeLog(findQueue(queueID),

  myMapLocalObjectIDToName(LongSwap(jobInfo->client.id)),

  tmp,gtmp,s);

  return NO_Q_JOB;

  }

return cC;

}



int clientCleanupFunction(void *p)

{

return 0;

}



/* cleanup function:

   deregisters library and restores nut */



void cleanup(void)

{

if (libHandle)

  DeregisterLibrary(libHandle);

if (handle)

  NWSRestoreNut(handle);

if (CLIBScreenID)

  DestroyScreen(CLIBScreenID);

NWDSFreeContext(dContext);

}



int ListAction (LONG keyPressed, LIST **elementSelected, LONG *itemNumber, void *listParameter)

{

QINFO *qi;

char tmp[16];

enum {L_QN, L_QI, L_JD, L_PC, L_PA=5, L_FP, L_BM };





elementSelected = elementSelected;

listParameter = listParameter;





if (keyPressed == M_ESCAPE)

  return 1;



qi=&queues[*itemNumber];



NWSPushList(handle);

NWSInitForm(handle);



NWSAppendCommentField (L_QN, 1, "Queue name:", handle);

NWSAppendCommentField (L_QN, 15, qi->queueName, handle);

NWSAppendCommentField (L_PA, 1, "Patch active:", handle);

NWSAppendBoolField (L_PA, 15, NORMAL_FIELD, &qi->patchCall, NULL, handle);

NWSAppendCommentField (L_FP, 1, "Filter PS:", handle);

NWSAppendBoolField (L_FP, 15, NORMAL_FIELD, &qi->filterPS, NULL, handle);

NWSAppendCommentField (L_BM, 1, "Banner mode:     (0=keep, 1=force, 2=no banner)", handle);

NWSAppendIntegerField (L_BM, 15, NORMAL_FIELD, &qi->bannerMode, KEEP_BANNER,

NO_BANNER,NULL,handle);

NWSAppendCommentField (L_JD, 1, "Jobs deleted:", handle);

NWSAppendIntegerField (L_JD, 15, PROMPT_FIELD, &qi->jobsDeleted,0,0x7fffffff,NULL,handle);

NWSAppendCommentField (L_PC, 1, "Patch count:", handle);

NWSAppendIntegerField (L_PC, 15, PROMPT_FIELD, &qi->callCount,0,0x7fffffff,NULL,handle);

NWSAppendCommentField (L_QI, 1, "Queue ID:", handle);

sprintf(tmp,"%08lX",qi->queueID);

NWSAppendCommentField (L_QI, 15, tmp,handle);



inForm=1;

NWSEditPortalForm (MSG_FORM, 11, 40, 10, max(60,20+maxLen), F_NOVERIFY, F_NO_HELP, 3, handle);

NWSDestroyForm (handle);

NWSPopList(handle);

inForm=0; NWSUpdatePortal(myPortal);

return -1;

}



/* hook for NWQBeginJobService:

   - force banner page

   - if filterPS==1, remove non PS jobs

*/



int NWQbeginJobService (LONG queueID, LONG jobType, NWQJobRec_t *jobInfo, int *handle)

{

int cC;

QINFO *q;



cC=NWQBeginJobService (queueID, jobType, jobInfo, handle);

if ((q=findQueue(queueID))==NULL)

  return cC;

if (!q->patchCall)

  return cC;

if (cC)

  return cC;

switch (q->bannerMode) {

  case KEEP_BANNER: break;

  case NO_BANNER:

  ((NWQPrintServerRec_t*)jobInfo->jobServerRecord)->printControlFlags&=BANNER_OFF;

  break;

  case FORCE_BANNER:

  ((NWQPrintServerRec_t*)jobInfo->jobServerRecord)->printControlFlags|=BANNER_ON;

  break;

  }

q->callCount++;

if (q->filterPS)

  if (!isPSjob(*handle))

    if ((cC=removeJob(queueID, jobInfo, handle))==NO_Q_JOB)

      q->jobsDeleted++;

return cC;

}



void main(int argc,char *argv[])

{

int     i;

LONG    ccode, infoPortalNumber;

PCB     *ipPtr;

WORD screenHeight, screenWidth;



dContext = NWDSCreateContext( );



if (argc>1)

  escapeMode=!strcmp(argv[1],"esc") ? M_ESCAPE : 0;



if (dContext==ERR_CONTEXT_CREATION)

  {

  ConsolePrintf("Error creating context\n");

  return;

  }

if (ccode=readParams())

  {

  ConsolePrintf("Error %x reading %s\n",ccode,MYINI);

  return;

  }



if (queueCnt==0)

  {

  ConsolePrintf("No valid parameters in %s\n",MYINI);

  NWDSFreeContext(dContext);

  return;

  }



NLMHandle=GetNLMHandle();



CLIBScreenID = CreateScreen("XATPSENH Screen", AUTO_DESTROY_SCREEN);

if (!CLIBScreenID)

  return;

allocTag = AllocateResourceTag(NLMHandle, "XATPSENH Alloc Tag",AllocSignature);



if (!allocTag)

  {

  DestroyScreen(CLIBScreenID);

  return;

  }



if (NWSInitializeNut(MSG_NAME,MSG_VER,NORMAL_HEADER,NUT_REVISION_LEVEL,

    messages, 0, CLIBScreenID, allocTag, &handle))

        {

        DestroyScreen(CLIBScreenID);

        return;

        }



AtUnload(cleanup);



DisplayScreen(CLIBScreenID);

GetSizeOfScreen (&screenHeight, &screenWidth);



infoPortalNumber = NWSCreatePortal(14l,0l,11l,(LONG)screenWidth,

                        9l,(LONG)screenWidth-2, SAVE,

                        NULL,/* "XATPSENH Status",*/

                        VNORMAL, SINGLE, VNORMAL, CURSOR_OFF, VIRTUAL, handle);



if (infoPortalNumber > MAXPORTALS)

  {

  printf ("Unable to create information portal\n");

  DestroyScreen(CLIBScreenID);

  return;

  }





NWSGetPCB (&ipPtr, infoPortalNumber, handle);

myPortal=ipPtr;

NWSClearPortal(myPortal);

NWSUpdatePortal(myPortal);



if ((libHandle=RegisterLibrary(clientCleanupFunction))!=-1)

  {

  ConsolePrintf("Library registered\n");

  run=1;

  AtUnload(cleanup);

  }



writeLog(NULL,"XATPSENH started",NULL,NULL,0);

NWSInitList (handle, free);



for (i=0;i<queueCnt;i++)

  NWSAppendToList (queues[i].queueName,(void *) 0, handle);



NWSList (MSG_LIST, 5, 40, queueCnt, max(48,5+maxLen),

                M_NO_SORT | escapeMode  | M_SELECT , NULL, handle, NULL, ListAction, 0);



NWSDestroyList (handle);



NWSDestroyPortal (infoPortalNumber, handle);

cleanup();

}



